#include "OptFileParser.h"
#include <fstream>
#include "Disk.h"

#ifdef WIN32
#pragma warning(disable: 4127)
#endif

bool OptFileParser::readCurve(string option, Curve &curve, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		curve.Load(optVals);
		return true;
	}

	return false;
}

bool OptFileParser::readIndex(string option, Indices &indices, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{
		ENSURE(optVals.size() == 1);

		int matched;
		int index;
		matched = sscanf(optVals[0].c_str(), "%i", &index);
		ENSURE(matched == 1);
		indices.insert(index);

		return true;
	}

	return false;
}

bool OptFileParser::readImgShape(string option, CShape &imgShape, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{
		ENSURE(optVals.size() == 3);

		int matched;
		matched = sscanf(optVals[0].c_str(), "%i", &imgShape.width);
		ENSURE(matched == 1);
		matched = sscanf(optVals[1].c_str(), "%i", &imgShape.height);
		ENSURE(matched == 1);
		matched = sscanf(optVals[2].c_str(), "%i", &imgShape.nBands);
		ENSURE(matched == 1);

		return true;
	}

	return false;
}

bool OptFileParser::readProbShape(string option, GS_P_Shape &probShape, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{
		probShape = GS_P_Shape::Default();
		return readImgShape(option, probShape.imgShape, optStr, optVals);
	}

	return false;
}

bool OptFileParser::readLevelsType(string option, GS_ProblemLevels::LevelsType &levelsType, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);

		if(optVals[0].compare("LT_DOB") == 0)
		{
			levelsType = GS_ProblemLevels::LT_DOB;
		}
		else if(optVals[0].compare("LT_DOG") == 0)
		{
			levelsType = GS_ProblemLevels::LT_DOG;
		}
		else
		{
			REPORT_FAILURE("Unknown levels type - %s.", optVals[0].c_str());
		}
		
		return true;
	}

	return false;
}


bool OptFileParser::readEnum(string option, int& enumVal, string optStr, vector<string> &optVals, const char** enumStrs)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);

		int iEnum = 0;
		while(enumStrs[iEnum] != NULL)
		{
			if(optVals[0].compare(enumStrs[iEnum]) == 0)
			{
				enumVal = iEnum;
				return true;
			}
			iEnum++;
		}

		REPORT_FAILURE("Unknown %s type - %s\n", option.c_str(), optVals[0].c_str());
	}

	return false;

}


bool OptFileParser::readFilterType(string option, Filter_Driver::FT_Type& fltType, string optStr, vector<string>& optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);

		if(optVals[0].compare("FT_NULL") == 0)
		{
			fltType = Filter_Driver::FT_NULL;
		}
		else if(optVals[0].compare("FT_SHARPEN") == 0)
		{
			fltType = Filter_Driver::FT_SHARPEN;
		}
		else if(optVals[0].compare("FT_ELE") == 0)
		{
			fltType = Filter_Driver::FT_ELE;
		}
		else if(optVals[0].compare("FT_TEST") == 0)
		{
			fltType = Filter_Driver::FT_TEST;
		}
		else if(optVals[0].compare("FT_RELIGHT") == 0)
		{
			fltType = Filter_Driver::FT_RELIGHT;
		}
		else if(optVals[0].compare("FT_DEBLOCK") == 0)
		{
			fltType = Filter_Driver::FT_DEBLOCK;
		}
		else if(optVals[0].compare("FT_MOTION_BLUR") == 0)
		{
			fltType = Filter_Driver::FT_MOTION_BLUR;
		}		
		else if(optVals[0].compare("FT_GRAD_CURVE") == 0)
		{
			fltType = Filter_Driver::FT_GRAD_CURVE;
		}
		else if(optVals[0].compare("FT_NPR") == 0)
		{
			fltType = Filter_Driver::FT_NPR;
		}
		else if(optVals[0].compare("FT_TM") == 0)
		{
			fltType = Filter_Driver::FT_TM;
		}
		else if(optVals[0].compare("FT_COLORIZE") == 0)
		{
			fltType = Filter_Driver::FT_COLORIZE;
		}
		else
		{
			REPORT_FAILURE("Unknown filter type - %s.", optVals[0].c_str());
		}
		
		return true;
	}

	return false;
}

bool OptFileParser::readAppOp(string option, AppOperation &appOP, string optStr, vector<string> &optVals)
{	
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);

		if(optVals[0].compare("OP_FILTER") == 0)
		{
			appOP = OP_FILTER;
		}
		else if(optVals[0].compare("OP_FILTER_VIDEO") == 0)
		{
			appOP = OP_FILTER_VIDEO;
		}
		else if(optVals[0].compare("OP_FILTER_VIDEO_CR") == 0)
		{
			appOP = OP_FILTER_VIDEO_CR;
		}
		else if(optVals[0].compare("OP_TEST_FILTER") == 0)
		{
			appOP = OP_TEST_FILTER;
		}
		else if(optVals[0].compare("OP_GENERIC") == 0)
		{
			appOP = OP_GENERIC;
		}
		else
		{
			REPORT_FAILURE("Unknown app operation - %s.", optVals[0].c_str());
		}
		
		return true;
	}

	return false;
}


bool OptFileParser::readBool(string option, bool &boolTarget, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);

		if(optVals[0].compare("true") == 0)
		{
			boolTarget = true;
		}
		else if(optVals[0].compare("false") == 0)
		{
			boolTarget = false;
		}
		else
		{
			REPORT_FAILURE("Invalid bool val - %s", optVals[0].c_str());

		}
		return true;
	}

	return false;
}

bool OptFileParser::readInt(string option, int &intTarget, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		int matched = sscanf(optVals[0].c_str(), "%i", &intTarget);
		ENSURE(matched == 1);

		return true;
	}

	return false;
}

bool OptFileParser::readUint(string option, uint &uintTarget, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		int matched = sscanf(optVals[0].c_str(), "%u", &uintTarget);
		ENSURE(matched == 1);

		return true;
	}

	return false;
}


bool OptFileParser::readFloat(string option, float &floatTarget, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		int matched = sscanf(optVals[0].c_str(), "%f", &floatTarget);
		ENSURE(matched == 1);

		return true;
	}

	return false;
}

bool OptFileParser::readDouble(string option, double &doubleTarget, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		int matched = sscanf(optVals[0].c_str(), "%lf", &doubleTarget);
		ENSURE(matched == 1);

		return true;
	}

	return false;
}


bool OptFileParser::addString(string option, vector<string> &strArray, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		strArray.push_back(optVals[0]);
		return true;
	}

	return false;
}

bool OptFileParser::addInt(string option, vector<int> &intArray, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		int param;
		int matched = sscanf(optVals[0].c_str(), "%i", &param);
		ENSURE(matched == 1);
		intArray.push_back(param);
		return true;
	}

	return false;
}


bool OptFileParser::addBool(string option, vector<bool> &boolArray, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		bool boolVal;
		bool matched = readBool(option, boolVal, optStr, optVals);
		ENSURE(matched == true);
		boolArray.push_back(boolVal);
		return true;
	}

	return false;
}

bool OptFileParser::addFloat(string option, vector<float> &floatArray, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		float param;
		int matched = sscanf(optVals[0].c_str(), "%f", &param);
		ENSURE(matched == 1);
		floatArray.push_back(param);
		return true;
	}

	return false;
}

bool OptFileParser::addDirName(string option, vector<string> &strArray, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		string dirStr = optVals[0];
		if(dirStr[dirStr.size() - 1] != OS_PATH_DELIMIT)
		{
			dirStr += OS_PATH_DELIMIT_STR;
		}		
		strArray.push_back(dirStr);
		return true;
	}

	return false;
}

bool OptFileParser::addProbFilter(string option, vector<GS_Problem::Filter>& filters, string optStr, vector<string>& optVals)
{
	if(optStr.compare(option) == 0)
	{	
		int optValCount = (int) optVals.size();
		ENSURE(optValCount >= 1);
		ProbFilterType fltType;
		vector<string> enumVal;
		enumVal.push_back(optVals[0]);
		bool enumSuccess = readEnum(option, (int&) fltType, optStr, enumVal, ProbFilterTypeStr);
		INSIST(enumSuccess == true);

		GS_Problem::Filter newFilter = GS_Problem::Filter::Default(fltType);
		int maxParams = (int) newFilter.params.floatList.size();
		INSIST(optValCount - 1 <= maxParams);
		for(int iFltParam = 1; iFltParam < optValCount; iFltParam++)
		{
			float fltParam;
			int matched = sscanf(optVals[iFltParam].c_str(), "%f", &fltParam);
			ENSURE(matched == 1);
			newFilter.params.floatList[iFltParam - 1] = fltParam;
		}
		
		filters.push_back(newFilter);		
		return true;
	}

	return false;
}

bool OptFileParser::readString(string option, string &strTarget, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		strTarget = optVals[0];
		return true;
	}

	return false;
}

bool OptFileParser::readDirName(string option, string &strTarget, string optStr, vector<string> &optVals)
{
	if(optStr.compare(option) == 0)
	{			
		ENSURE(optVals.size() == 1);
		strTarget = optVals[0];
		if(strTarget[strTarget.size() - 1] != OS_PATH_DELIMIT)
		{
			strTarget += OS_PATH_DELIMIT_STR;
		}		
		return true;
	}

	return false;
}


void OptFileParser::neutralizePaths(string &line)
{
	for(uint i = 0; i < line.length(); i++)
	{
		if(line[i] == WRONG_OS_PATH_DELIMIT)
		{
			line[i] = OS_PATH_DELIMIT;
		}
	}
}

void OptFileParser::fixImgFNs(OptFileParams& params)
{
	INSIST(params.imgRange.second >= params.imgRange.first);
	if(params.useDefaultFNs == false)
	{
		if(params.imgFNs.size() > 0)
		{
			IntPair range         = params.imgRange;
			vector<string> imgFNs = params.imgFNs;
			if((range.first == -1) && (range.second == -1))
			{
				range.first  = 0;
				range.second = (int) params.imgFNs.size() - 1;
			}
			CHECK_RANGE(range.first,  0, (int) imgFNs.size() - 1);
			CHECK_RANGE(range.second, 0, (int) imgFNs.size() - 1);

			params.imgFNs.clear();
			for(int iImg = range.first; iImg <= range.second; iImg++)
			{
				params.imgFNs.push_back(imgFNs[iImg]);
			}
		}
	}
	else
	{		
		INSIST(params.imgFNs.size() == 0);

		char fileName[1024];
		IntPair range = params.imgRange;
		INSIST(range.first >= 0);
		for(int iImg = range.first; iImg <= range.second; iImg++)
		{
			sprintf(fileName, "%svid-%03i.tga", params.inDir.c_str(), iImg);
			params.imgFNs.push_back(fileName);
		}
	}

	params.useDefaultFNs = false;
	params.imgRange = IntPair(-1, -1);
}

void OptFileParser::fixOutDirs(OptFileParams& params)
{
	params.fltParams.gssParams.outDir = params.outDir;
	params.gsvsParams.outDir          = params.outDir;
	Disk::MakeDir(params.outDir);
}

void OptFileParser::fixVS_Params(OptFileParams& params)
{
	*((GS_Solver::Params*) &params.gsvsParams) = params.fltParams.gssParams;
	params.gsvsParams.writeProb = params.gsvsParams.writeSol = true;
}

OptFileParser::OptFileParams OptFileParser::Load(const char* optionsFileName, char* argv[], bool dumpOptionsRead)
{
	OptFileParams params = OptFileParams::Default();

	vector<string> optStrings;

	ifstream inputStream(optionsFileName);
	if(!inputStream.is_open())
	{
		REPORT_FAILURE("Couldn't open file - %s\n", optionsFileName);
	}
	ENSURE(inputStream.is_open());
	string lineBuff;
	while(std::getline(inputStream, lineBuff))
	{
		optStrings.push_back(lineBuff);
	}
	inputStream.close();

	int argI = 0;
	while(argv[argI] != NULL)
	{
		char charBuff[1024];
		sprintf(charBuff, "%s", argv[argI]);
		string lineBuff(charBuff);
		optStrings.push_back(lineBuff);
		argI++;
	}
    	
	for(uint iOptStr = 0; iOptStr < optStrings.size(); iOptStr++)
	{
		vector<string> lineTokens;	
		string lineBuff = optStrings[iOptStr];

		neutralizePaths(lineBuff);

		if(lineBuff.size() == 0) continue;
		if(lineBuff[0] == '#')   continue;

		lineTokens.clear();
		Tokenize(lineBuff, lineTokens, " \t=");
		if(lineTokens.size() == 0) continue;

		if(lineTokens.size() < 2)
		{
			PRINT_INT(iOptStr);
			PRINT_STRING(lineTokens[0]);
			ENSURE(lineTokens.size() >= 2);
		}		

		string optStr = lineTokens[0];
		lineTokens.erase(lineTokens.begin(), lineTokens.begin() + 1);

		bool optStrMatched = false;

		optStrMatched |= readInt(     "seed",                 params.seed,                   optStr, lineTokens);
		optStrMatched |= readAppOp(   "appOP",                params.appOP,                  optStr, lineTokens);
		optStrMatched |= readDirName( "outDir",               params.outDir,                 optStr, lineTokens);
		optStrMatched |= readDirName( "inDir",                params.inDir,                  optStr, lineTokens);
		optStrMatched |= readInt(     "startIndex",           params.imgRange.first,         optStr, lineTokens);
		optStrMatched |= readInt(     "endIndex",             params.imgRange.second,        optStr, lineTokens);
		optStrMatched |= readBool(    "useDefaultFNs",        params.useDefaultFNs,          optStr, lineTokens);
		optStrMatched |= addString(   "imgFN",                params.imgFNs,                 optStr, lineTokens);

		if(optStrMatched == true) continue;

		optStrMatched |= readGSP_Params(params.gspParams, optStr, lineTokens);
		if(optStrMatched == true) continue;	

		optStrMatched |= readFltParams(params.fltParams, optStr, lineTokens);
		if(optStrMatched == true) continue;	

		optStrMatched |= readGenericParams(params.genericParams, optStr, lineTokens);
		if(optStrMatched == true) continue;

		optStrMatched |= readGSVS_Params(params.gsvsParams, optStr, lineTokens);
		if(optStrMatched == true) continue;

		if(optStrMatched == false)
		{
			REPORT_FAILURE("Unknown option: %s\n", optStr.c_str());
		}
	}

	fixImgFNs(params);
	fixOutDirs(params);
	fixVS_Params(params);

	if(dumpOptionsRead == true)
	{
		params.Dump();
	}
	
	return params;
}

bool OptFileParser::readFltParams(Filter_Driver::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string FLT_Marker = "FLT.";

	if(beginsWith(optStr, FLT_Marker))
	{
		optStr = trimStart(optStr, (uint) FLT_Marker.size());

		optStrMatched |= readGSP_Params(params.gspParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;	

		optStrMatched |= readGSS_Params(params.gssParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;	

		optStrMatched |= readFltSharpenParams(params.gsf_SharpenParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;	

		optStrMatched |= readFltEleParams(params.gsf_EleParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;

		optStrMatched |= readFltMotionBlurParams(params.gsf_motionBlurParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;

		optStrMatched |= readFltNprParams(params.gsf_nprParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;

		optStrMatched |= readFltGradCurveParams(params.gsf_gradCurveParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;

		optStrMatched |= readFltToneMapParams(params.gsf_tmParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;

		optStrMatched |= readFltColorizeParams(params.gsf_clzParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;

		optStrMatched |= readFltTestParams(params.gsf_testParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;	

		optStrMatched |= readFltDeblockParams(params.gsf_deblockParams, optStr, lineTokens);
		if(optStrMatched == true) return optStrMatched;			

		optStrMatched |= readFilterType("type", params.type, optStr, lineTokens);
	}	

	return optStrMatched;
}

bool OptFileParser::readGSVS_Params(GS_VSolver::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "GSVS.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());

		optStrMatched |= readDouble("fidelity_tGrad",   params.fidelity_tGrad,   optStr, lineTokens);
		optStrMatched |= readDouble("flowCnfPow",       params.flowCnfPow,       optStr, lineTokens);
		optStrMatched |= readBool(  "useFlow",          params.useFlow,          optStr, lineTokens);
		optStrMatched |= readBool(  "useFlowCnf",       params.useFlowCnf,       optStr, lineTokens);
		optStrMatched |= readInt(   "inCoreFrameLimit", params.inCoreFrameLimit, optStr, lineTokens);
		optStrMatched |= readInt(   "outOfCoreIters",   params.outOfCoreIters,   optStr, lineTokens);
	}

	return optStrMatched;
}

bool OptFileParser::readGSP_Params(GS_Problem::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string GSP_Marker = "GSP.";
	if(beginsWith(optStr, GSP_Marker))
	{
		optStr = trimStart(optStr, GSP_Marker.size());

		optStrMatched |= readEnum(      "channelsType", (int&) params.channelsType,   optStr, lineTokens, ChannelsTypeStr);
		optStrMatched |= readLevelsType("levelsType",   params.plsParams.levelsType,  optStr, lineTokens);
		optStrMatched |= readInt(       "levelCount",   params.plsParams.levelCount,  optStr, lineTokens);
		optStrMatched |= readFloat(     "sigmaS",       params.plsParams.sigmaS,      optStr, lineTokens);
		optStrMatched |= readFloat(     "sigmaD",       params.plsParams.sigmaD,      optStr, lineTokens);
		optStrMatched |= readFloat(     "sigmaS_mult",  params.plsParams.sigmaS_mult, optStr, lineTokens);
		optStrMatched |= readFloat(     "sigmaD_mult",  params.plsParams.sigmaD_mult, optStr, lineTokens);

		optStrMatched |= addProbFilter("postProcess.type", params.postprocessFilters, optStr, lineTokens);
		optStrMatched |= addProbFilter("preProcess.type",  params.preprocessFilters,  optStr, lineTokens);

		optStrMatched |= readProbShape( "probShape",   params.problemShape,          optStr, lineTokens);
	}

	return optStrMatched;
}


bool OptFileParser::readGSS_Params(GS_Solver::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "GSS.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());

		optStrMatched |= readDouble("accuracy",       params.accuracy,           optStr, lineTokens);
		optStrMatched |= readInt(   "maxIters",       params.maxIters,           optStr, lineTokens);
		optStrMatched |= readInt(   "maxLevels",      params.maxLevelsInPrecond, optStr, lineTokens);
		optStrMatched |= readBool(  "useGlobalDC",    params.useGlobalDatacost,  optStr, lineTokens);
		optStrMatched |= readBool(  "useSDIW",        params.useSDI_weighing,    optStr, lineTokens);
		optStrMatched |= readDouble("sdiNorm",        params.sdiNorm,            optStr, lineTokens);
		optStrMatched |= readDouble("pinchExp",       params.pinchFuncExp,       optStr, lineTokens);
		optStrMatched |= readDouble("pinchScale",     params.pinchFuncScale,     optStr, lineTokens);
		optStrMatched |= readDouble("fidelity_data",  params.fidelity_data,      optStr, lineTokens);
		optStrMatched |= readDouble("fidelity_sGrad", params.fidelity_sGrad,     optStr, lineTokens);
		optStrMatched |= readIndex( "levelToSolve",   params.levelsToSolve,      optStr, lineTokens);
		optStrMatched |= readIndex( "channelToSolve", params.channelsToSolve,    optStr, lineTokens);
		optStrMatched |= readBool(  "writeProb",      params.writeProb,          optStr, lineTokens);
		optStrMatched |= readBool(  "writeSol",       params.writeSol,           optStr, lineTokens);
	}

	return optStrMatched;
}

bool OptFileParser::readFltSharpenParams(GSF_Sharpen::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "Sharpen.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readFloat("mult", params.multiplier, optStr, lineTokens);
	}

	return optStrMatched;
}


bool OptFileParser::readFltEleParams(GSF_ELE::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "ELE.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readFloat("mult",   params.multiplier,  optStr, lineTokens);
		optStrMatched |= readFloat("elNorm", params.edgeLenNorm, optStr, lineTokens);		
	}

	return optStrMatched;
}

bool OptFileParser::readFltMotionBlurParams(GSF_MotionBlur::Params&  params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "MB.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readFloat("minW",     params.minGradWeight, optStr, lineTokens);
		optStrMatched |= readFloat("cnfScale", params.flowCnfScale,  optStr, lineTokens);		
		optStrMatched |= readFloat("lenScale", params.flowLenScale,  optStr, lineTokens);		
	}

	return optStrMatched;
}

bool OptFileParser::readFltNprParams(GSF_NPR::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "NPR.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readFloat("lenNorm",  params.edgeLenNorm,  optStr, lineTokens);
		optStrMatched |= readCurve("lenCurve", params.edgeLenCurve, optStr, lineTokens);		
	}

	return optStrMatched;
}

bool OptFileParser::readFltGradCurveParams(GSF_GradCurve::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "GC.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readCurve("gcurve", params.gradCurve, optStr, lineTokens);		
	}

	return optStrMatched;
}


bool OptFileParser::readFltToneMapParams(GSF_ToneMap::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "TM.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readFloat("afScale",      params.attenFuncScale, optStr, lineTokens);
		optStrMatched |= readFloat("afExp",        params.attenFuncExp,   optStr, lineTokens);		
		optStrMatched |= readInt(  "minLevelSize", params.minLevelSize,   optStr, lineTokens);		
	}

	return optStrMatched;
}

bool OptFileParser::readFltColorizeParams(GSF_Colorize::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "CLZ.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readBool("improvedSDI", params.improvedSDI,     optStr, lineTokens);
		optStrMatched |= readDouble("edgeSen",   params.edgeSensitivity, optStr, lineTokens);
		optStrMatched |= readFloat("bfSigS",     params.bfSigS,          optStr, lineTokens);
		optStrMatched |= readFloat("bfSpaceS",   params.bfSpaceS,        optStr, lineTokens);
	}

	return optStrMatched;	
}

bool OptFileParser::readFltTestParams(GSF_Test::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "TEST.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readFloat("lightAng",      params.lightDircAng,  optStr, lineTokens);
		optStrMatched |= readFloat("lightStrength", params.lightStrength, optStr, lineTokens);
	}

	return optStrMatched;
}

bool OptFileParser::readFltRelightParams(GSF_Relight::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "RELIGHT.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readFloat("lightAng",      params.lightDircAng,  optStr, lineTokens);
		optStrMatched |= readFloat("lightStrength", params.lightStrength, optStr, lineTokens);
	}

	return optStrMatched;
}

bool OptFileParser::readFltDeblockParams(GSF_Deblock::Params& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "DEBLOCK.";
	if(beginsWith(optStr, Marker))
	{
		optStr = trimStart(optStr, Marker.size());
		optStrMatched |= readCurve("bgCurve", params.blockGradCurve, optStr, lineTokens);
		optStrMatched |= readFloat("uvMult",  params.uvMult,         optStr, lineTokens);
	}

	return optStrMatched;
}

bool OptFileParser::readGenericParams(GenericOpts& params, string optStr, vector<string> lineTokens)
{
	bool optStrMatched = false;

	string Marker = "Generic.";
	if(beginsWith(optStr, Marker))	
	{
		optStr = trimStart(optStr, Marker.size());

		optStrMatched |= addString( "string1", params.strList1,  optStr, lineTokens);
		optStrMatched |= addString( "string2", params.strList2,  optStr, lineTokens);
		optStrMatched |= addInt(    "int1",    params.intList1,  optStr, lineTokens);
		optStrMatched |= addInt(    "int2",    params.intList2,  optStr, lineTokens);
		optStrMatched |= addFloat(  "float",   params.floatList, optStr, lineTokens);
	}

	return optStrMatched;
}



