#ifndef __GS_PROBLEM_H__
#define __GS_PROBLEM_H__

#include "GS_Problem_Defs.h"
#include "GS_ProblemLevel.h"
#include "GS_ProblemLevels.h"
#include "GS_ProblemInput.h"
#include "GenericOpts.h"

typedef enum
{
	PFT_SATURATE_COLOR = 0,
	PFT_SHARPEN,
	PFT_DENOISE,
	PFT_LUM2RGB,
	PFT_ROBUST_RESCALE,
	PFT_EDGE_VIS1,
	PFT_EDGE_VIS2,
	PFT_LAST = PFT_EDGE_VIS2
} ProbFilterType;
const extern char* ProbFilterTypeStr[PFT_LAST + 2];

class GS_Problem
{
public:	

	typedef struct _Filter
	{
		ProbFilterType type;
		GenericOpts params;

		static _Filter Default(ProbFilterType _type);
		void Write(ofstream &outStream, string paramPrefix) const;
		void Dump(string prefix) const;		
	} Filter;

	typedef struct _FilterAuxData
	{
		EdgeMetrics emData;
		string imgFN;
	} FilterAuxData;

	typedef struct _Params
	{
		ChannelsType channelsType;
		vector<Filter> preprocessFilters;
		vector<Filter> postprocessFilters;
		GS_P_Shape problemShape;
		GS_ProblemLevels::Params plsParams;
		static _Params Default();
		void Dump(string id);
		void Write(string id) const;
		static _Params Load(string id);
	}Params;

private:
	//depends - write, read, load, extractChannel, 
	Params params;
	GS_ProblemLevels  problemLevels;
	GS_ProblemInput   input;
	GS_Image          result;
	vector<Callback>  resultUpdate_Callbacks;

public:
	GS_Problem(Params _params = Params::Default());
	GS_Problem(GS_ProblemInput& _input, Params _params = Params::Default());
	void Resize(GS_P_Shape shape, int levelCount = -1);
	void ResetInput(const GS_ProblemInput &_input);
	void ResetLevels();
	void ResetResidual(const Indices &levelsNotToUse);
	void UpdateResult(const Indices &levelsToUse);	

	void Copy(const GS_Problem& cpySrc, int srcChannel);
	GS_Problem ExtractChannel(int iChannel);

	GS_ProblemInput* GetInputBuffer();	
	Params GetParams();
	GS_ProblemLevel* GetLevel(int iLevel);		
	GS_Image GetResult();
	GS_Image GetResultRGB(bool postProcessResult = true) const;

	int GetLevelCount()
	{
		return this->problemLevels.GetLevelCount();
	}

	void Add_resultUpdate_callback(Callback callback);

private:
	void verifyChannelsTypeCorrectness(bool tolerateUnknown = false);
	void make_resultUpdate_callbacks();

public: //GS_Problem-IO.cpp
	string GetResultImgFN(string id, string fileExt) const;
	static string getFN( string id, string suffix, string fileExt);

	void Write(   string id, string fileExt = "tga", Indices levelsToUse = Indices());
	void WriteSol(string id, string fileExt = "tga", bool postProcessSol = true) const;
	void Read(    string id, string fileExt = "raw");

	static GS_Problem Load(string id);
	static GS_P_Shape GetProbShape(string id);


public: //GS_Problem-Filter.cpp
	void PreProcessInput();
	static void FilterImg(GS_Image& img, const vector<Filter>& filters, const FilterAuxData &auxData =  FilterAuxData());
private:
	void preProcessInput(GS_Image& img);
};

#endif

