#include "GS_ProblemLevel.h"
#include "ImageIO.h"
#include "Disk.h"

void GS_ProblemLevel::Write(string id, int levelFileID, string fileExt) const
{
	string baseDir = Disk::GetBaseDirName(id);
	string baseID  = Disk::GetBaseFileName(id);

	bool rawMode = false;
	if(fileExt.compare("raw") == 0)
		rawMode = true;

	char fileName[1024];

	sprintf(fileName, "%sorg-%02i-%s.%s",   baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	if(rawMode)
		ImageIO::WriteFile(this->org, fileName);
	else
		ImageIO::WriteFile(ColorHelper::ConvertChannels(CT_RGB, this->org), fileName);
																			
	sprintf(fileName, "%sflt-%02i-%s.%s",   baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	if(rawMode)
		ImageIO::WriteFile(this->flt, fileName);
	else
		ImageIO::WriteFile(ColorHelper::ConvertChannels(CT_RGB, this->flt), fileName);


	sprintf(fileName, "%sdata-%02i-%s.%s",    baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	if(rawMode)
		ImageIO::WriteFile(this->data, fileName);
	else
		ImageIO::WriteFile(ColorHelper::ConvertChannels(CT_RGB, this->data), fileName);
																			
	sprintf(fileName, "%sdataW-%02i-%s.%s",   baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	ImageIO::WriteFile(this->dataW, fileName);
																			
	string img1FN, img2FN;
	bool normalizeGrads = (fileExt.compare("raw") == 0) ? false : true;	
	vector<GS_Image> imgs;
	vector<string>   imgFNs;
																			
	sprintf(fileName, "%sorgGx-%02i-%s.%s", baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	imgFNs.push_back(fileName);
	sprintf(fileName, "%sfltGx-%02i-%s.%s", baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	imgFNs.push_back(fileName);
	imgs.push_back(this->orgGF_X);
	imgs.push_back(this->fltGF_X);
	WriteGradImgs(imgs, imgFNs, normalizeGrads);

	imgs.clear(); imgFNs.clear();

	sprintf(fileName, "%sorgGy-%02i-%s.%s", baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	imgFNs.push_back(fileName);
	sprintf(fileName, "%sfltGy-%02i-%s.%s", baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	imgFNs.push_back(fileName);	
	imgs.push_back(this->orgGF_Y);
	imgs.push_back(this->fltGF_Y);
	WriteGradImgs(imgs, imgFNs, normalizeGrads);
}

void GS_ProblemLevel::WriteGradImgs(const vector<GS_Image>& imgs, const vector<string>& imgFNs, bool normalizeBeforeDump) const
{
	int imgCount = (int) imgs.size();
	INSIST(imgCount > 0);
	INSIST(imgCount == (int) imgFNs.size());

	if(normalizeBeforeDump == false)
	{
		for(int iImg = 0; iImg < imgCount; iImg++)
		{
			ImageIO::WriteFile(imgs[iImg], imgFNs[iImg]);
		}
	}
	else
	{
		float normDenom = 0.0f;
		float stdDevScale = 10;

		for(int iImg = 0; iImg < imgCount; iImg++)
		{
			CShape imgShape = imgs[iImg].Shape();
			GS_Image magImg(imgShape);
			ImageProcessing::Operate(magImg, imgs[iImg], 0.0f, ImageProcessing::OP_ABS);
			for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++)
			{
				float mean, stdDev;
				ImageProcessing::GetMeanAndDev(magImg, iChannel, mean, stdDev);
				float currNormDenom = (mean + (stdDev * stdDevScale)) / 2.0f;
				SET_IF_GT(normDenom, currNormDenom);
			}
		}

		for(int iImg = 0; iImg < imgCount; iImg++)
		{
			GS_Image normImg = imgs[iImg].Clone();
			ImageProcessing::Operate(normImg, normImg, normDenom, ImageProcessing::OP_DIV);
			ImageProcessing::Operate(normImg, normImg, 0.5f,      ImageProcessing::OP_ADD);
			ImageIO::WriteFile(normImg, imgFNs[iImg]);
		}
	}
}



void GS_ProblemLevel::Read(string id, int levelFileID, string fileExt)
{
	string baseDir = Disk::GetBaseDirName(id);
	string baseID  = Disk::GetBaseFileName(id);

	char fileName[1024];

	CShape expectedImgShape = this->org.Shape();

	sprintf(fileName, "%sorg-%02i-%s.%s",   baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	ImageIO::ReadFile(this->org, fileName);									
																			
	sprintf(fileName, "%sflt-%02i-%s.%s",   baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	ImageIO::ReadFile(this->flt, fileName);									

	sprintf(fileName, "%sdata-%02i-%s.%s",    baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	ImageIO::ReadFile(this->data, fileName);									
																			
	sprintf(fileName, "%sdataW-%02i-%s.%s",   baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	ImageIO::ReadFile(this->dataW, fileName);
																			
	sprintf(fileName, "%sorgGx-%02i-%s.%s", baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	ImageIO::ReadFile(this->orgGF_X, fileName);								
																			
	sprintf(fileName, "%sfltGx-%02i-%s.%s", baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	ImageIO::ReadFile(this->fltGF_X, fileName);								
																			
	sprintf(fileName, "%sorgGy-%02i-%s.%s", baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	ImageIO::ReadFile(this->orgGF_Y, fileName);								
																			
	sprintf(fileName, "%sfltGy-%02i-%s.%s", baseDir.c_str(), levelFileID, baseID.c_str(), fileExt.c_str());
	ImageIO::ReadFile(this->fltGF_Y, fileName);

	INSIST(expectedImgShape == this->org.Shape());
	INSIST(expectedImgShape == this->flt.Shape());
	INSIST(expectedImgShape == this->orgGF_X.Shape());
	INSIST(expectedImgShape == this->orgGF_Y.Shape());
	INSIST(expectedImgShape == this->fltGF_X.Shape());
	INSIST(expectedImgShape == this->fltGF_Y.Shape());
}

