#include "GS_ProblemLevel.h"
#include "ImageProcessing.h"
#include "ImageIO.h"

GS_ProblemLevel::GS_ProblemLevel()
{
}

void GS_ProblemLevel::Resize(CShape imgShape)
{
	this->org.ReAllocate(imgShape);
	this->flt.ReAllocate(imgShape);

	this->data.ReAllocate(imgShape);
	this->dataW.ReAllocate(imgShape);

	this->orgGF_X.ReAllocate(imgShape);
	this->orgGF_Y.ReAllocate(imgShape);

	this->fltGF_X.ReAllocate(imgShape);
	this->fltGF_Y.ReAllocate(imgShape);
}

//bug
void GS_ProblemLevel::ComputeGrads()
{
	CShape imgShape = this->org.Shape();
	INSIST(imgShape.PixelCount() > 0 );

	int pixelAddr       = 0;
	int leftPixelAddr   = -imgShape.nBands;
	int bottomPixelAddr = -(imgShape.nBands * imgShape.width);
	int rightPixelAddr  = imgShape.nBands;
	int topPixelAddr    = (imgShape.nBands * imgShape.width);
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++)
	for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++, 
		pixelAddr++, leftPixelAddr++, bottomPixelAddr++, 
		rightPixelAddr++, topPixelAddr++)
	{
		imgType currPixel = this->org[pixelAddr];
		//bug
		if(x > 0)
			orgGF_X[pixelAddr] = currPixel - this->org[leftPixelAddr];
		else
			//orgGF_X[pixelAddr] = 0;
			//orgGF_X[pixelAddr] = currPixel - this->org[rightPixelAddr];
			orgGF_X[pixelAddr] = currPixel - this->org.Pixel(x + 1, y, iChannel);
			//orgGF_X[pixelAddr] = currPixel - this->org.Pixel(imgShape.width - 1, y, iChannel);

		//bug
		if(y > 0)
			orgGF_Y[pixelAddr] = currPixel - this->org[bottomPixelAddr];
		else
			//orgGF_Y[pixelAddr] = 0;
			//orgGF_Y[pixelAddr] = currPixel - this->org[topPixelAddr];
			orgGF_Y[pixelAddr] = currPixel - this->org.Pixel(x, y + 1, iChannel);
			//orgGF_Y[pixelAddr] = currPixel - this->org.Pixel(x, imgShape.height - 1, iChannel);
	}

	ImageProcessing::CopyChannels(this->fltGF_X, this->orgGF_X);
	ImageProcessing::CopyChannels(this->fltGF_Y, this->orgGF_Y);
}

//void GS_ProblemLevel::ComputeGrads()
//{
//	CShape imgShape = this->org.Shape();
//	INSIST(imgShape.PixelCount() > 0 );
//
//	int pixelAddr       = 0;
//	int leftPixelAddr   = -imgShape.nBands;
//	int bottomPixelAddr = -(imgShape.nBands * imgShape.width);
//	int rightPixelAddr  = imgShape.nBands;
//	int topPixelAddr    = (imgShape.width * imgShape.nBands);
//	for(int y = 0; y < imgShape.height; y++)
//	for(int x = 0; x < imgShape.width;  x++)
//	for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++, pixelAddr++, rightPixelAddr++, topPixelAddr++, leftPixelAddr++, bottomPixelAddr++)
//	{
//		imgType currPixel = this->org[pixelAddr];
//
//		if((x < imgShape.width - 1) && (x > 0))
//			orgGF_X[pixelAddr] = this->org[rightPixelAddr] - this->org[pixelAddr];
//		else
//			orgGF_X[pixelAddr] = 0;
//
//		if((y < imgShape.height - 1) && (y > 0))
//			orgGF_Y[pixelAddr] = this->org[topPixelAddr] - this->org[pixelAddr];
//		else
//			orgGF_Y[pixelAddr] = 0;
//	}
//
//	ImageProcessing::CopyChannels(this->fltGF_X, this->orgGF_X);
//	ImageProcessing::CopyChannels(this->fltGF_Y, this->orgGF_Y);
//}


void GS_ProblemLevel::GetData(GS_Image& _org,     GS_Image& _flt,
							  GS_Image& _data,    GS_Image& _dataW,
							  GS_Image& _orgGF_X, GS_Image& _orgGF_Y,
							  GS_Image& _fltGF_X, GS_Image& _fltGF_Y)
{
	_data  = this->data;
	_dataW = this->dataW;

	GetData(_org, _flt, _orgGF_X, _orgGF_Y, _fltGF_X, _fltGF_Y);
}


void GS_ProblemLevel::GetData(GS_Image& _org,     GS_Image& _flt,
							  GS_Image& _orgGF_X, GS_Image& _orgGF_Y,
							  GS_Image& _fltGF_X, GS_Image& _fltGF_Y)
{
	_org = this->org;
	_flt = this->flt;
	GetDataGF(_orgGF_X, _orgGF_Y, _fltGF_X, _fltGF_Y);
}



void GS_ProblemLevel::GetDataGF(GS_Image& _orgGF_X, GS_Image& _orgGF_Y,
								GS_Image& _fltGF_X, GS_Image& _fltGF_Y)
{
	_orgGF_X = this->orgGF_X;
	_orgGF_Y = this->orgGF_Y;
	_fltGF_X = this->fltGF_X;
	_fltGF_Y = this->fltGF_Y;
}

void GS_ProblemLevel::Copy(const GS_ProblemLevel& cpySrc, int srcChannel)
{
	INSIST(this->org.Shape().nBands == 1);

	ImageProcessing::CopyChannel(this->org,     cpySrc.org,     0, srcChannel);
	ImageProcessing::CopyChannel(this->flt,     cpySrc.flt,     0, srcChannel);
	ImageProcessing::CopyChannel(this->data,    cpySrc.data,    0, srcChannel);
	ImageProcessing::CopyChannel(this->dataW,   cpySrc.dataW,   0, srcChannel);
	ImageProcessing::CopyChannel(this->orgGF_X, cpySrc.orgGF_X, 0, srcChannel);
	ImageProcessing::CopyChannel(this->orgGF_Y, cpySrc.orgGF_Y, 0, srcChannel);
	ImageProcessing::CopyChannel(this->fltGF_X, cpySrc.fltGF_X, 0, srcChannel);
	ImageProcessing::CopyChannel(this->fltGF_Y, cpySrc.fltGF_Y, 0, srcChannel);
}

