#include "GS_ProblemLevels.h"

GS_ProblemLevels::Params GS_ProblemLevels::Params::Default()
{
	Params params;
	params.levelCount  = 1;
	params.levelsType  = LT_DEFAULT;
	params.sigmaD      = 1.0f;	
	params.sigmaS      = 0.1f;
	params.sigmaD_mult = 1.0f;
	params.sigmaS_mult = 1.0f;
	return params;
}

void GS_ProblemLevels::Params::Dump(string prefix)
{
	const char* prefixPtr = prefix.c_str();

	printf("\n");
	printf("%s GS Problem levels params:\n", prefixPtr);

	printf("%s levels - %i\n", prefixPtr, this->levelCount);
	printf("%s levels type - ", prefixPtr);

	switch(this->levelsType)
	{
	case LT_DOG:
		printf("%s Difference of Gaussians:\n", prefixPtr);
		printf("%s  sigmaD      - %f\n", prefixPtr, this->sigmaD);
		printf("%s  sigmaD_mult - %f\n", prefixPtr, this->sigmaD_mult);
		break;
	case LT_DOB:
		printf("%s Difference of Bilaterals:\n", prefixPtr);
		printf("%s  sigmaD      - %f\n", prefixPtr, this->sigmaD);
		printf("%s  sigmaD_mult - %f\n", prefixPtr, this->sigmaD_mult);
		printf("%s  sigmaS      - %f\n", prefixPtr, this->sigmaS);
		printf("%s  sigmaS_mult - %f\n", prefixPtr, this->sigmaS_mult);
		break;
	default:
		REPORT_FAILURE("Unknown levels type - %i\n", this->levelsType);
		break;
	}

	printf("\n");
}

void GS_ProblemLevels::Params::Write(ofstream &outStream, string paramPrefix) const
{
	outStream << paramPrefix << "levelCount = "  << this->levelCount  << "\n";
	outStream << paramPrefix << "sigmaD = "      << this->sigmaD      << "\n";
	outStream << paramPrefix << "sigmaS = "      << this->sigmaS      << "\n";
	outStream << paramPrefix << "sigmaD_mult = " << this->sigmaD_mult << "\n";
	outStream << paramPrefix << "sigmaS_mult = " << this->sigmaS_mult << "\n";

	outStream << paramPrefix << "levelsType = ";
	switch(this->levelsType)
	{
	case LT_DOG:
		outStream << "LT_DOG\n";
		break;
	case LT_DOB:
		outStream << "LT_DOB\n";
		break;
	default:
		REPORT_FAILURE("Unknown levels type - %i\n", this->levelsType);
		break;
	}
}

bool GS_ProblemLevels::Params::operator==(const _Params _params)
{
	return ((this->levelCount == _params.levelCount) &&
		    (this->levelsType == _params.levelsType) &&
			(fabs(this->sigmaD - _params.sigmaD) <= FLT_EPSILON) && 
			(fabs(this->sigmaS - _params.sigmaS) <= FLT_EPSILON) &&
			(fabs(this->sigmaD_mult - _params.sigmaD_mult) <= FLT_EPSILON) && 
			(fabs(this->sigmaS_mult - _params.sigmaS_mult) <= FLT_EPSILON));
}

