#ifndef  __GS_FOURIER_SOLVER_H__
#define  __GS_FOURIER_SOLVER_H__

#include "GS_Solver.h"
#include "fftw3.h"

class GS_FourierSolver
{
public:
	typedef struct _Params : public GS_Solver::Params
	{		
		static _Params Default();
		void Dump(string prefix);

		_Params()
		{
		}

		_Params(GS_Solver::Params _params)
		{
			(*this) = Default();
			*((GS_Solver::Params*) this) = _params;
		}
	} Params;

private:
	Params      params;
	GS_Problem* problem;
	GS_P_Shape  problemShape;

	float* fftBuff;
	float* ftLapY;
	float* ftLapX;
	fftwf_plan fftPlan;	

public:
	GS_FourierSolver(GS_Problem* _problem, Params params = Params::Default());
	GS_FourierSolver();
	~GS_FourierSolver();
	void Solve();

	Params GetParams()
	{
		return this->params;
	}

	void SetParams(Params _params)
	{
		this->params = _params;
		adjustParams();
	}
	
private:
	void adjustParams();
	void initFFTW();
	void destroyFFTW();
	void solve(int iLevel);
	void fftwSolve(CFloatImage& imgData, CFloatImage& imgGradX, CFloatImage& imgGradY, float dataCost);
};

#endif
