#include "GS_Solver.h"

void GS_Solver::Params::Dump(string prefix)
{
	const char* prefixPtr = prefix.c_str();

	printf("\n");
	printf("%s GS Solver params:\n", prefixPtr);

	printf("%s  Levels to solve:\n", prefixPtr);
	if(this->levelsToSolve.size() == 0)
		printf("%s\t  All\n", prefixPtr);
	else
	{
		for(Indices::const_iterator iLevel = levelsToSolve.begin();
			iLevel != levelsToSolve.end();
			iLevel++)
		{		
			printf("%s\t  %i\n", prefixPtr, *iLevel);
		}
	}

	printf("%s  Channels to solve:\n", prefixPtr);
	if(this->channelsToSolve.size() == 0)
		printf("%s\t  All\n", prefixPtr);
	else
	{
		for(Indices::const_iterator iChannel = channelsToSolve.begin();
			iChannel != channelsToSolve.end();
			iChannel++)
		{
			printf("%s\t  %i\n", prefixPtr, *iChannel);
		}
	}

	printf("%s  (accuracy, maxIters, maxLevels) - (%e, %i, %i)\n", prefixPtr, this->accuracy,       this->maxIters, this->maxLevelsInPrecond);
	printf("%s  pinch (scale, exp)              - (%e, %e)\n",     prefixPtr, this->pinchFuncScale, this->pinchFuncExp);
	printf("%s  fidelty (data, sGrad)           - (%e, %e)\n",     prefixPtr, this->fidelity_data,  this->fidelity_sGrad);
	printf("%s  useGlobalDatacost               - %i\n",           prefixPtr, this->useGlobalDatacost);
	printf("%s  useSDI_weighing                 - %i\n",           prefixPtr, this->useSDI_weighing);
	printf("%s  sdiNorm                         - %e\n",           prefixPtr, this->sdiNorm);
	printf("%s  (writeProb, writeSol)           - (%i, %i)\n",     prefixPtr, this->writeProb,      this->writeSol);
	if(this->writeProb || this->writeSol)
	{
		printf("%s  outDir                          - %s\n", prefixPtr, this->outDir.c_str());
	}
	printf("\n");
}

GS_Solver::Params GS_Solver::Params::Default()
{
	Params params;
	params.accuracy           = 1E-06;
	params.maxIters           = 50;
	params.maxLevelsInPrecond = 4;
	params.fidelity_data      = 3E-02;
	params.fidelity_sGrad     = 1.0;

	params.pinchFuncScale   = 1.0;
	params.pinchFuncExp     = 5.0;
	
	params.useGlobalDatacost = true;
	params.useSDI_weighing   = false;
	params.sdiNorm           = 75.0;

	params.writeProb = false;
	params.writeSol  = false;
	params.outDir    = "." + OS_PATH_DELIMIT_STR;

	return params;
}

