#include "GS_VSolver.h"

void GS_VSolver::Params::Dump(string prefix)   
{
	const char* prefixPtr = prefix.c_str();

	printf("\n");
	printf("%s GS VSolver params:\n", prefixPtr);

	INSIST(this->writeProb == true);
	INSIST(this->writeSol  == true);
	GS_Solver::Params::Dump(prefix);

	printf("%s  fidelty tGrad               - %e\n",       prefixPtr, this->fidelity_tGrad);
	printf("%s  (useFlow, useFlowCnf)       - (%i, %i)\n", prefixPtr, this->useFlow, this->useFlowCnf);
	if(this->useFlowCnf)
		printf("%s  flowCnfPow                  - %.02f\n", prefixPtr, this->flowCnfPow);
	
	printf("%s  useConstraint (left, right) - (%i, %i)\n", prefixPtr, this->useLeftConstraint, this->useRightConstraint);	
	if(this->useLeftConstraint || this->useRightConstraint)
		printf("%s  constraintFN  (left, right) - (%s, %s)\n", prefixPtr, this->leftConstraintImgFN.c_str(), this->rightConstraintImgFN.c_str());

	if(this->inCoreFrameLimit != -1)
	{
		printf("%s  inCoreFrameLimit            - %i\n", prefixPtr, this->inCoreFrameLimit);
		printf("%s  outOfCoreIters              - %i\n", prefixPtr, this->outOfCoreIters);
	}

	printf("%s  Images:\n", prefixPtr);
	for(vector<string>::const_iterator imgFN = imgFNs.begin();
		imgFN != imgFNs.end();
		imgFN++)
	{
		printf("%s  \t %s:\n", prefixPtr, imgFN->c_str());
	}

	printf("\n");
}

GS_VSolver::Params GS_VSolver::Params::Default()
{
	Params params;
	*((GS_Solver::Params*) &params) = GS_Solver::Params::Default();
	
	params.writeProb = params.writeSol = true;
	
	params.fidelity_tGrad = 1.0;

	params.useFlow = params.useFlowCnf = true;
	params.flowCnfPow = 1.0;

	params.useLeftConstraint = params.useRightConstraint = false;

	params.inCoreFrameLimit = -1;
	params.outOfCoreIters   = 3;

	return params;
}
