#include "SparseSolver_LasPack.h"
#include "Timer.h"

SparseSolver_LasPack::SparseSolver_LasPack(size_v _varCount, SparseSolverParams _params)
{
	this->varCount  = _varCount;
	this->params    = _params;

	Boolean symmetry = (this->params.symmetric == true) ? True : False;
	Q_Constr(&(this->aMat), (char*) "A", this->varCount, symmetry, Rowws, Normal, True);
	V_Constr(&(this->xVec), (char*) "x", this->varCount, Normal, True);
	V_Constr(&(this->bVec), (char*) "b", this->varCount, Normal, True);	

	this->aMatPtr = &(this->aMat);
	this->xVecPtr = &(this->xVec);
	this->bVecPtr = &(this->bVec);

	errCheck();
}

SparseSolver_LasPack::~SparseSolver_LasPack() 
{
	Q_Destr(this->aMatPtr);
	V_Destr(this->xVecPtr);
	V_Destr(this->bVecPtr);

	errCheck();
}

void SparseSolver_LasPack::Solve(double accuracy, int maxIter, bool verbose) 
{
	Timer timer;
	if(verbose)
	{
		REPORT("Solving System...\n");
	}
	
	SetRTCAccuracy(accuracy); 

	CGIter(this->aMatPtr, 
		   this->xVecPtr, 
		   this->bVecPtr, 
		   maxIter, 
		   SSORPrecond, 
		   1.2);

	if(verbose)
	{
		REPORT("\tDone - %.3e in %.2f(s).\n", GetLastAccuracy(), timer.GetSecondsCount());
	}

	errCheck();
}

void SparseSolver_LasPack::DumpSystem()
{
	REPORT("System: %s\n", Q_GetName(aMatPtr));		
	for(size_v iRow = 1; iRow <= this->varCount; iRow++)
	{
		printf("[ ");
		for(size_v iCol = 1; iCol <= this->varCount; iCol++)
		{
			printf("%.02f ", Q_GetEl(aMatPtr, iRow, iCol));
		}
		printf("][ %.02f ] = [ %.02f ]\n", 
			V_GetCmp(xVecPtr, iRow), 
			V_GetCmp(bVecPtr, iRow));
	}
	errCheck();
}

bool SparseSolver_LasPack::IsSymetric()
{
	for(size_v iRow = 1; iRow < this->varCount; iRow++)
	{		
		for(size_v iCol = 1; iCol < this->varCount; iCol++)
		{
			double a  = Q_GetEl(aMatPtr,  iRow, iCol);
			double aa = Q_GetEl(aMatPtr,  iCol, iRow);
			errCheck();
			if(aa != a)
			{
				return false;
			}
		}			
	}
	return true;
}

