#pragma once

#include "Definitions.h"
#include "laspack/qmatrix.h"
#include "laspack/errhandl.h"
#include "laspack/itersolv.h"
#include "laspack/rtc.h"
#include "Debug.h"
#include "SparseSolver.h"
#include "LinearSolverDefs.h"

class SparseSolver_LasPack
{  
  size_v  varCount;

  QMatrix aMat;
  QMatrix *aMatPtr;

  Vector  xVec;
  Vector  *xVecPtr;

  Vector  bVec;
  Vector  *bVecPtr;

  SparseSolverParams params;

private:
	void errCheck()
	{
		LASErrIdType lasERR = LASResult();
		if(lasERR != LASOK)
		{
			WriteLASErrDescr(stdout);
			REPORT_FAILURE("LASPACK failure id: %i", lasERR);
		}
	}

public:

	void SetRow(size_v iRow, size_vv elemCount, precise rowX, precise rowB, const size_v* elemIDs, const precise* elemCoefs) 
	{
		ENSURE(iRow >= 0);
		ENSURE(iRow < this->varCount);

		Q_SetLen(this->aMatPtr, iRow + 1, elemCount);
		V_SetCmp(this->xVecPtr, iRow + 1, rowX);
		V_SetCmp(this->bVecPtr, iRow + 1, rowB);
		for(size_vv iElem = 0; iElem < elemCount; iElem++)
		{
			ENSURE(elemIDs[iElem] >= 0);
			ENSURE(elemIDs[iElem] < this->varCount);
			if(this->params.symmetric == true) 
			{
				ENSURE(elemIDs[iElem] >= iRow);
			}
			Q_SetEntry(this->aMatPtr, iRow + 1, iElem, elemIDs[iElem] + 1, elemCoefs[iElem]);
		}
		
		errCheck();
	}

	void SetRow(size_v iRow, size_vv elemCount, precise rowX, precise rowB, const vector<size_v>& elemIDs, const vector<precise>& elemCoefs) 
	{
		ENSURE(iRow >= 0);
		ENSURE(iRow < this->varCount);

		Q_SetLen(this->aMatPtr, iRow + 1, elemCount);
		V_SetCmp(this->xVecPtr, iRow + 1, rowX);
		V_SetCmp(this->bVecPtr, iRow + 1, rowB);
		for(size_v iElem = 0; iElem < elemCount; iElem++)
		{
			ENSURE(elemIDs[iElem] >= 0);			
			ENSURE(elemIDs[iElem] < this->varCount);
			if(this->params.symmetric == true) 
			{
				ENSURE(elemIDs[iElem] >= iRow);
			}
			Q_SetEntry(this->aMatPtr, iRow + 1, iElem, elemIDs[iElem] + 1, elemCoefs[iElem]);
		}
		
		errCheck();
	}

	double GetSolution(int iRow)
	{
		double sol = V_GetCmp(this->xVecPtr, iRow + 1);
		errCheck();
		return sol;
	}

	SparseSolver_LasPack(size_v _varCount, SparseSolverParams _params = SparseSolverParams::Default());

	~SparseSolver_LasPack();

	void DumpSystem();

	void Solve(double accuracy = 1e-5, int maxIter = 100, bool verbose = true);

	bool IsSymetric();
};

