#include "LinearSolver.h"
#include "SparseSqrSymMat.h"


void LinearSolver::SolveRandSys_GD(int size)
{
	SparseSqrSymMat mat1;
	precise* xOrg  = new precise[size];
	precise* xSol1 = new precise[size];
	precise* xSol2 = new precise[size];
	precise* b     = new precise[size];
	INSIST(xOrg  != NULL);
	INSIST(xSol1 != NULL);
	INSIST(xSol2 != NULL);
	INSIST(b     != NULL);

	mat1 = SparseSqrSymMat::GenRandMat(size);	
	SparseSymMat_CSR* mat2 = mat1.ConvertToCSR();

	Vec::FillRandValues(size, xOrg);
	mat1.Multiply(size, size, xOrg, b);

	Vec::FillRandValues(size, xSol1);
	Vec::Copy(size, xSol2, xSol1);

	precise err1 = LinearSolver::GetResidual(size, &mat1, xSol1, b);
	precise err2 = LinearSolver::GetResidual(size,  mat2, xSol2, b);
	INSIST(fabs(err1 - err2) < 0.0001);

	PRINT_SCI(fabs(err1 - err2));
	
	LinearSolver::Params params = LinearSolver::Params::Default();
	LinearSolver::Solve_CG(size, &mat1, xSol1, b, params);
	LinearSolver::Solve_CG(size,  mat2, xSol2, b, params);	

	mat1.DumpMatlab("A");	
	Vec::DumpMatlab(size, xOrg, "xIn");
	Vec::DumpMatlab(size, xSol1, "xSol1");
	Vec::DumpMatlab(size, xSol2, "xSol2");
	Vec::DumpMatlab(size, b, "b");

	delete [] xOrg;
	delete [] xSol1;
	delete [] xSol2;
	delete [] b;
	delete mat2;
}

