#include "LinearSolver.h"
#include "SparseSqrSymMat.h"

precise LinearSolver::GetResidual(int n, AbstractMat *A, precise *x, precise *b)
{
	precise *Ax = new precise[n]; //Av holds the vec result from A * x

	//r = b - Ax
	precise residual = 0.0;
	A->Multiply(n, n, x, Ax);  
	//#pragma omp parallel for reduction(+:residual)
	for(int i = 0; i < n; i++)
	{
		precise err = b[i] - Ax[i];
		residual += err * err;
	}

	delete [] Ax;	

	return residual;
}


LinearSolRet 
LinearSolver::Solve_CG(size_v n, AbstractMat* A, precise* x, precise* b, 
	     			   Params params)
{
#ifdef __USE_MKL__
	return Solve_PCG_mkl(n, A, x, b, params, NULL);	
#else
	return Solve_CG_pro(n, A, x, b, params);	
#endif
}


LinearSolRet 
LinearSolver::Solve_PCG(size_v n, AbstractMat* A, precise* x, precise* b, 
				  	    Params params, AbsractPreconditioner* MInv)
{
#ifdef __USE_MKL__
	return Solve_PCG_mkl(n, A, x, b, params, MInv);	
#else
	return Solve_PCG_pro(n, A, x, b, params, MInv);	
#endif
}

