#include "PrecondHBF.h"

PrecondHBF::PrecondHBF(size_v _width, size_v _height, int _levels)
{
	this->width  = _width;
	this->height = _height;
	this->levels = _levels;

	this->nodeCount = this->width * this->height;
	INSIST(this->nodeCount > 0);

	int maxLevels = PrecondHBF::maxNumberOfLevels(this->width, this->height);
	if(this->levels < 0)
	{
		this->levels = maxLevels;
	}
	else
	{
		this->levels = min(this->levels, maxLevels);
	}

	this->levels = min(this->levels, 4);
}


int PrecondHBF::maxNumberOfLevels(size_v _width, size_v _height)
{
	INSIST(_width  > 0);
	INSIST(_height > 0);

	int minPow2 = -1; // 2^minPow2 <= _width,_height
	while((_width > 0) && (_height > 0))
	{
		_width  >>= 1;
		_height >>= 1;
		minPow2++;		
	}

	int levels = minPow2 - 3;
	levels = max(levels, 0);
	return levels;
}

void PrecondHBF::sweepUp(precise* vec) const
{
	for(int iLevel = 0; iLevel <= this->levels - 2; iLevel++)
	{
		sweepUp(vec, iLevel);
	}		
}

void PrecondHBF::sweepDown(precise* vec) const
{
	for(int iLevel = this->levels - 2; iLevel >= 0; iLevel--)
	{
		sweepDown(vec, iLevel);
	}	
}

void PrecondHBF::sweepUp(precise* vec, int sourceLevel) const
{
	size_v neighStep = 1 << sourceLevel;
	size_v nodeStep  = neighStep << 1;

	precise oneHalf   = (precise) 0.5;
	precise oneQuater = (precise) 0.25;

	//#pragma omp parallel for
	for(size_v y = neighStep; y < this->height - neighStep; y += nodeStep)
	for(size_v x = neighStep; x < this->width  - neighStep; x += nodeStep)	
	{
		precise me = oneQuater * vec[nodeIndex(x,y)];
		vec[nodeIndex(x - neighStep, y)] += me;
		vec[nodeIndex(x + neighStep, y)] += me;
		vec[nodeIndex(x, y - neighStep)] += me;
		vec[nodeIndex(x, y + neighStep)] += me;
	}

	//#pragma omp parallel for
	for(size_v y = neighStep; y < this->height - neighStep; y += nodeStep)
	for(size_v x = 0;         x < this->width;              x += nodeStep)	
	{
		precise me =  oneHalf * vec[nodeIndex(x,y)];
		vec[nodeIndex(x, y - neighStep)] += me;
		vec[nodeIndex(x, y + neighStep)] += me;
	}

	//#pragma omp parallel for
	for(size_v y = 0;         y < this->height;             y += nodeStep)
	for(size_v x = neighStep; x < this->width  - neighStep; x += nodeStep)	
	{
		precise me = oneHalf * vec[nodeIndex(x,y)];
		vec[nodeIndex(x - neighStep, y)] += me;
		vec[nodeIndex(x + neighStep, y)] += me;
	}
}

void PrecondHBF::sweepDown(precise* vec, int targetLevel) const
{
	size_v neighStep = 1 << targetLevel;
	size_v nodeStep  = neighStep << 1;

	precise oneHalf   = (precise) 0.5;
	precise oneQuater = (precise) 0.25;

	//#pragma omp parallel for
	for(size_v y = neighStep; y < this->height - neighStep; y += nodeStep)
	for(size_v x = 0;         x < this->width;              x += nodeStep)	
	{
		vec[nodeIndex(x,y)] += oneHalf * vec[nodeIndex(x, y - neighStep)] +
			                   oneHalf * vec[nodeIndex(x, y + neighStep)];
	}

	//#pragma omp parallel for
	for(size_v y = 0;         y < this->height;             y += nodeStep)
	for(size_v x = neighStep; x < this->width  - neighStep; x += nodeStep)	
	{
		vec[nodeIndex(x,y)] += oneHalf * vec[nodeIndex(x - neighStep, y)] +
			                   oneHalf * vec[nodeIndex(x + neighStep, y)];
	}

	//#pragma omp parallel for
	for(size_v y = neighStep; y < this->height - neighStep; y += nodeStep)
	for(size_v x = neighStep; x < this->width  - neighStep; x += nodeStep)	
	{	
		vec[nodeIndex(x,y)] += oneQuater * vec[nodeIndex(x - neighStep, y)] +
							   oneQuater * vec[nodeIndex(x + neighStep, y)] + 
							   oneQuater * vec[nodeIndex(x, y - neighStep)] +
							   oneQuater * vec[nodeIndex(x, y + neighStep)];
	}
}




//void PrecondHBF::sweepUp(precise* vec, int sourceLevel) const
//{
//	size_v neighStep = 1 << sourceLevel;
//	size_v nodeStep  = neighStep << 1;
//
//	precise oneHalf   = (precise) 0.5;
//	precise oneQuater = (precise) 0.25;
//
//	for(size_v y = neighStep; y < this->height - neighStep; y += nodeStep)
//	for(size_v x = neighStep; x < this->width  - neighStep; x += nodeStep)	
//	{
//		precise me = oneQuater * vec[nodeIndex(x,y)];
//		vec[nodeIndex(x - neighStep, y)] += me;
//		vec[nodeIndex(x + neighStep, y)] += me;
//		vec[nodeIndex(x, y - neighStep)] += me;
//		vec[nodeIndex(x, y + neighStep)] += me;
//	}
//
//	for(size_v y = neighStep; y < this->height - neighStep; y += nodeStep)
//	for(size_v x = 0;         x < this->width;              x += nodeStep)	
//	{
//		precise me =  oneHalf * vec[nodeIndex(x,y)];
//		vec[nodeIndex(x, y - neighStep)] += me;
//		vec[nodeIndex(x, y + neighStep)] += me;
//	}
//
//	for(size_v y = 0;         y < this->height;             y += nodeStep)
//	for(size_v x = neighStep; x < this->width  - neighStep; x += nodeStep)	
//	{
//		precise me = oneHalf * vec[nodeIndex(x,y)];
//		vec[nodeIndex(x - neighStep, y)] += me;
//		vec[nodeIndex(x + neighStep, y)] += me;
//	}
//}
//
//void PrecondHBF::sweepDown(precise* vec, int targetLevel) const
//{
//	size_v neighStep = 1 << targetLevel;
//	size_v nodeStep  = neighStep << 1;
//
//	precise oneHalf   = (precise) 0.5;
//	precise oneQuater = (precise) 0.25;
//
//	for(size_v y = neighStep; y < this->height - neighStep; y += nodeStep)
//	for(size_v x = 0;         x < this->width;              x += nodeStep)	
//	{
//		vec[nodeIndex(x,y)] += oneHalf * vec[nodeIndex(x, y - neighStep)] +
//			                   oneHalf * vec[nodeIndex(x, y + neighStep)];
//	}
//
//	for(size_v y = 0;         y < this->height;             y += nodeStep)
//	for(size_v x = neighStep; x < this->width  - neighStep; x += nodeStep)	
//	{
//		vec[nodeIndex(x,y)] += oneHalf * vec[nodeIndex(x - neighStep, y)] +
//			                   oneHalf * vec[nodeIndex(x + neighStep, y)];
//	}
//
//	for(size_v y = neighStep; y < this->height - neighStep; y += nodeStep)
//	for(size_v x = neighStep; x < this->width  - neighStep; x += nodeStep)	
//	{
//		vec[nodeIndex(x,y)] += oneQuater * vec[nodeIndex(x - neighStep, y)] +
//			                   oneQuater * vec[nodeIndex(x + neighStep, y)] + 
//							   oneQuater * vec[nodeIndex(x, y - neighStep)] +
//			                   oneQuater * vec[nodeIndex(x, y + neighStep)];
//	}
//}
