#include "PrecondHBF_stack.h"


PrecondHBF_stack::PrecondHBF_stack(size_v _totalVarCount, vector<CShape> imgShapes, int _maxLevels)
{
	INSIST(imgShapes.size() > 0);

	this->totalVarCount = 0;
	this->imgCount = (uint) imgShapes.size();
	for(uint iImg = 0; iImg < this->imgCount; iImg++)
	{
		CShape currShape = imgShapes[iImg];
		PrecondHBF currPrecond(currShape.width, currShape.height, _maxLevels);
		this->preconditioners.push_back(currPrecond);

		size_v varsInCurrImg = currShape.width * currShape.height;
		INSIST(varsInCurrImg > 0);
		this->totalVarCount += varsInCurrImg;
		this->varsInImg.push_back(varsInCurrImg);
	}
	INSIST(this->totalVarCount == _totalVarCount);
}

void 
PrecondHBF_stack::Apply(size_v n, precise* vec) const
{
	INSIST(n == this->totalVarCount);

	precise* currImgVec = vec;
	for(uint iImg = 0; iImg < this->imgCount; iImg++)
	{
		size_v varsInCurrImg = this->varsInImg[iImg];
		this->preconditioners[iImg].Apply(varsInCurrImg, currImgVec);
		currImgVec += varsInCurrImg;
	}
}


