#include "SparseSqrSymMat.h"

void 
SparseSqrSymMat::Dump()
{
	printf("Matrix (%i x %i):\n", this->n, this->n);

	for(size_v iRow = 0; iRow < this->n; iRow++)
	for(size_v iCol = 0; iCol < this->n; iCol++)
	{		
		if(iCol != this->n - 1)
			printf("%0.2f, ", (float) GetElem(iRow, iCol));
		else
			printf("%0.2f\n", (float) GetElem(iRow, iCol));
	}
}


void 
SparseSqrSymMat::DumpMatlab(const char* matName)
{
	printf("%s = [ ", matName);

	for(size_v iRow = 0; iRow < this->n; iRow++)
	for(size_v iCol = 0; iCol < this->n; iCol++)
	{		
		if(iCol != this->n - 1)
			printf("%1.4e ",  (double) GetElem(iRow, iCol));
		else
			printf("%1.4e; ", (double) GetElem(iRow, iCol));
	}

	printf("]\n");
}


void 
SparseSqrSymMat::DumpRandVecMult(size_v size)
{
	SparseSqrSymMat mat;
	precise* x = new precise[size];
	precise* b = new precise[size];
	INSIST(x != NULL);
	INSIST(b != NULL);

	mat = GenRandMat(size);
	mat.DumpMatlab();	

	Vec::FillRandValues(size, x);
	Vec::DumpMatlab(size, x);

	mat.Multiply(size, size, x, b);	
	Vec::DumpMatlab(size, b);

	delete [] x;
	delete [] b;
}


SparseSqrSymMat SparseSqrSymMat::GenRandMat(size_v size)
{
	SparseSqrSymMat mat(size);

	for(size_v i = 0; i < size; i++)
	{
		size_v rowElemCount = myRand() % (size + 1);
		mat.SetElem(i, i, genProb());
		for(size_v j = 0; j < rowElemCount; j++)
		{
			size_v iCol = myRand() % size;
			if(i < iCol)
				mat.SetElem(i, iCol, genProb());
		}
	}

	return mat;
}


SparseSqrSymMat SparseSqrSymMat::GenIdenMat(size_v size)
{
	SparseSqrSymMat mat(size);

	for(size_v i = 0; i < size; i++)
	{
		mat.SetElem(i, i, 1);
	}

	return mat;
}



SparseSymMat_CSR*
SparseSqrSymMat::ConvertToCSR()
{	
	size_vv nonZeroCount = this->GetNonZeroCount();
	SparseSymMat_CSR* csrMat = new SparseSymMat_CSR(this->n, this->n, nonZeroCount);

	size_v iRow = 0;
	for(vector<MatRow>::const_iterator currRow = this->matData.begin();
		currRow != this->matData.end();
		currRow++, iRow++)
	{
		INSIST(currRow->size() > 0);
		vector<precise> values;
		vector<size_v> cols;

		for(MatRow::const_iterator currCol = currRow->begin(); 
			currCol != currRow->end(); 
			currCol++)
		{
			size_v iCol = currCol->first;
			precise val = currCol->second;

			values.push_back(val);
			cols.push_back(iCol);
		}

		csrMat->SetRow(iRow, values, cols);
	}

	return csrMat;
}

