#pragma once

#include "Definitions.h"
#include "LinearSolverDefs.h"
#include "AbstractMat.h"
#include "Vec.h"

class SparseSymMat_CSR : public AbstractMat
{
	precise* values;
	size_v*  columns;
	size_vv* rowIndices;

	size_v rowCount, colCount;
	size_v nextRowIndex;

	size_vv nonZeroCount;
	size_vv nextNonZeroIndex;

public:
	SparseSymMat_CSR(size_v _rows, size_v _cols, size_vv nonZeroCountGuess);
	~SparseSymMat_CSR();

	void SetRow(size_v iRow, const precise* rowValues, const 
			    size_v* rowColumns, size_vv rowNonZeroCount);

	void SetRow(size_v iRow, const vector<precise>& rowValues, 
		        const vector<size_v>& rowColumns);

	void Multiply(size_v inLen, size_v outLen, 
				  const precise* inVec, precise* outVec) const;

	void Multiply_pro(size_v inLen, size_v outLen, 
					  const precise* inVec, precise* outVec) const;

	void Multiply_mkl(size_v inLen, size_v outLen, 
					  const precise* inVec, precise* outVec) const;


	size_v GetRowCount() const
	{
		return this->rowCount;
	}
	
	size_v GetColCount() const
	{
		return this->colCount;
	}	

private:
	void reallocNonZeroBuff(size_vv nonZeroCountGuess);
	void makeSpaceForNonZeros(size_vv rowNonZeroCount);


public: //SparseSymMat_CSR-Debug.cpp
	void DumpMatlab(char* matName = "A");


	friend class LinearSolver;
};


