#ifndef __VEC_H__
#define __VEC_H__

#include "Definitions.h"
#include "LinearSolverDefs.h"
#include "mkl_blas.h"

class Vec
{

public:
	static void Multiply(size_v n, precise* vec, precise scalar)
	{
		//#pragma omp parallel for
		for(size_v i = 0; i < n; i++)
		{
			vec[i] *= scalar;
		}
	}

	static precise Multiply(size_v n, precise* vec1, precise* vec2)
	{		
		precise productSum = 0;	

		//#pragma omp parallel for reduction(+:productSum) 
		for(size_v i = 0; i < n; i++)
		{
			productSum += vec1[i] * vec2[i];
		}

		//int incr = 1;
		//productSum = sdot(&n, vec1, &incr, vec2, &incr);

		return productSum;
	}

	static void AddEqual(size_v n, precise* sumVec, const precise* vecToAdd)
	{
		//#pragma omp parallel for
		//for(size_v i = 0; i < n; i++)
		//{
		//	sumVec[i] += vecToAdd[i];
		//}

		int incr = 1;
		float alpha = 1.0f;
		saxpy(&n, &alpha, vecToAdd, &incr, sumVec, &incr);
	}

	static void SetToNull(size_v n, precise* vec)
	{
		//#pragma omp parallel for
		//for(size_v i = 0; i < n; i++)
		//{
		//	vec[i] = 0;
		//}

		memset(vec, 0, sizeof(precise) * n);
	}

	static void Copy(size_v n, precise* targetVec, const precise* srcVec)
	{
		//#pragma omp parallel for
		//for(size_v i = 0; i < n; i++)
		//{
		//	targetVec[i] = srcVec[i];
		//}

		//int incr = 1;
		//scopy(&n, srcVec, &incr, targetVec, &incr);

		memcpy(targetVec, srcVec, sizeof(precise) * n);
	}

	static void DumpMatlab(size_v n, precise* vec, const char* vecName = "x")
	{
		printf("%s = [ ", vecName);
		for(size_v i = 0; i < n; i++)
		{
			printf("%1.4e; ", (double) vec[i]);
		}
		printf("]\n");
	}

	static void FillRandValues(size_v n, precise *vec)
	{
		//#pragma omp parallel for
		for(size_v i = 0; i < n; i++)
		{
			vec[i] = genProb();
		}
	}
};

#endif

