#pragma once

#include "Definitions.h"
#include "Debug.h"
#include "SparseSymMat_CSR.h"
#include "SparseSqrSymMat.h"
#include "SparseSolver.h"
#include "PrecondHBF_stack.h"


class SparseSolver_pro
{  
	size_v  varCount;
	SparseSymMat_CSR* aMat;
	precise* xVec;  
	precise* bVec;
	PrecondHBF_stack* precond;

	SparseSolverParams params;

public:

	SparseSolver_pro(size_v varCount, SparseSolverParams _params = SparseSolverParams::Default());

	~SparseSolver_pro();

	void Solve(double accuracy = 1e-5, int maxIter = 100, bool verbose = true);

	void SetRow(size_v iRow, size_vv elemCount, precise rowX, precise rowB, 
				const size_v* elemIDs, const precise* elemCoefs) 
	{
		ENSURE(iRow >= 0);
		ENSURE(iRow < this->varCount);

		this->xVec[iRow] = rowX;
		this->bVec[iRow] = rowB;
		aMat->SetRow(iRow, elemCoefs, elemIDs, elemCount);
	}

	void SetRow(size_v iRow, size_vv elemCount, precise rowX, precise rowB, 
		const vector<size_v>& elemIDs, const vector<precise>& elemCoefs) 
	{
		ENSURE(iRow >= 0);
		ENSURE(iRow < this->varCount);

		this->xVec[iRow] = rowX;
		this->bVec[iRow] = rowB;
		aMat->SetRow(iRow, elemCoefs, elemIDs);
	}

	precise GetSolution(size_v iRow)
	{
		ENSURE(iRow >= 0);
		ENSURE(iRow < this->varCount);

		return this->xVec[iRow];
	}
};


