#include "GSF_ELE.h"
#include "ImageIO.h"

GSF_ELE::GSF_ELE(GS_Problem* _problem, const Params &_params) : GS_Filter(_problem)
{
	this->params = _params;
}

void GSF_ELE::FilterLevel(int iLevel, void* userData)
{	
	INSIST(this->problem->GetInputBuffer()->IsEdgeMetricsDefined() == true);

	Params *levelParams        = (userData == NULL) ? &this->params : (Params *) userData;
	EdgeMetrics &edgeMetrics   = this->problem->GetInputBuffer()->emData;
	GS_ProblemLevel *levelProb = this->problem->GetLevel(iLevel);

	INSIST(levelParams->multiplier >= -1.0f);

	GS_Image levelOrgGF_X, levelOrgGF_Y;
	GS_Image levelFltGF_X, levelFltGF_Y;
	levelProb->GetDataGF(levelOrgGF_X, levelOrgGF_Y, levelFltGF_X, levelFltGF_Y);

	CShape imgShape = levelOrgGF_X.Shape();
	uint pixelAddr  = 0;
	uint nodeAddr   = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, nodeAddr++)
	{
		float edgeLen    = edgeMetrics.edgeLen[nodeAddr];
		float edgeAngRad = edgeMetrics.edgeDirc[nodeAddr];

		float gradWeightX = pow(cos(edgeAngRad), 2.0f);
		float gradWeightY = 1 - gradWeightX; 
		gradWeightX *= (edgeLen / levelParams->edgeLenNorm) * levelParams->multiplier;
		gradWeightY *= (edgeLen / levelParams->edgeLenNorm) * levelParams->multiplier;

		for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++, pixelAddr++)
		{
			levelFltGF_X[pixelAddr] = (imgType) (levelOrgGF_X[pixelAddr] + (levelOrgGF_X[pixelAddr] * gradWeightX));
			levelFltGF_Y[pixelAddr] = (imgType) (levelOrgGF_Y[pixelAddr] + (levelOrgGF_Y[pixelAddr] * gradWeightY));
		}
	}
}
