#include "GSF_MotionBlur.h"
#include "MathHelper.h"
#include "Transform-2x2.h"

GSF_MotionBlur::GSF_MotionBlur(GS_Problem* _problem, const Params &_params) : GS_Filter(_problem)
{
	this->params = _params;
	INSIST(this->problem->GetInputBuffer()->IsFlowFieldDefined() == true);
}

void GSF_MotionBlur::FilterLevel(int iLevel, void* userData)
{	
	Params* levelParams = (userData == NULL) ? &this->params : (Params*) userData;

	GS_ProblemLevel* levelProb = this->problem->GetLevel(iLevel);

	GS_Image levelOrgGF_X, levelOrgGF_Y;
	GS_Image levelFltGF_X, levelFltGF_Y;
	levelProb->GetDataGF(levelOrgGF_X, levelOrgGF_Y, levelFltGF_X, levelFltGF_Y);

	CFloatImage flowField = this->problem->GetInputBuffer()->flowField;
	CShape flowShape = flowField.Shape();
	INSIST(flowShape.nBands == 3);

	CShape imgShape = levelOrgGF_X.Shape();
	uint pixelAddr = 0;
	uint flowAddr  = 0;
	for(int y = 0; y < imgShape.height; y++)
	for(int x = 0; x < imgShape.width;  x++, flowAddr += flowShape.nBands)
	{
		float flowCnf = flowField[flowAddr + 2];
		FVector2 flowVec(flowField[flowAddr + 0], flowField[flowAddr + 1]);
		float flowVecLen = flowVec.Len();
		if(flowVecLen != 0.0f)
			flowVec = flowVec / flowVecLen;

		for(int iChannel = 0; iChannel < imgShape.nBands; iChannel++, pixelAddr++)
		{
			FVector2 gradVec(levelOrgGF_X[pixelAddr], levelOrgGF_Y[pixelAddr]);
			float gradVecLen = gradVec.Len();
			if(gradVecLen != 0.0f)
				gradVec = gradVec / gradVecLen;

			CHECK_RANGE(levelParams->minGradWeight, 0.0f, 1.0f);
			float gradVecMag = (1.0f - myAbs(gradVec.dot(flowVec))) * (1.0f - levelParams->minGradWeight);
			gradVecMag += FLT_EPSILON;
			gradVecMag = pow(gradVecMag, flowVecLen * levelParams->flowLenScale);
			gradVecMag = pow(gradVecMag, flowCnf    * levelParams->flowCnfScale);
			gradVecMag += levelParams->minGradWeight;
			SET_IF_LT(gradVecMag, 1.0);
			CHECK_RANGE(gradVecMag, 0.0f, 1.0f);

			levelFltGF_X[pixelAddr] = levelOrgGF_X[pixelAddr] * gradVecMag;
			levelFltGF_Y[pixelAddr] = levelOrgGF_Y[pixelAddr] * gradVecMag;
		}
	}
}


