#include "GSF_Null.h"
#include "ImageIO.h"
#include "Disk.h"

GS_Image GSF_Null::Null(GS_ProblemInput& probInput,
 				        GS_Problem::Params probParams,
					    GS_Solver::Params solParams)
{
	INCR_TAB_LEVEL();
	string paramsSpacing((TAB_SPACE_COUNT + 1) * TAB_LEVEL, ' ');
	
	REPORT("Creating GS problem... \n");
	probParams.Dump(paramsSpacing);
	GS_Problem prob(probInput, probParams);	

	REPORT("Filtering GS problem... \n");
	GSF_Null nullFlt(&prob);
	if(solParams.levelsToSolve.size() == 0)
		nullFlt.Filter();
	else
		nullFlt.Filter(solParams.levelsToSolve);

	REPORT("Solving GS problem... \n");
	solParams.Dump(paramsSpacing);
	GS_Solver solver(&prob, solParams);
	solver.Solve();

	DCR_TAB_LEVEL();
	return prob.GetResultRGB();			
}

void GSF_Null::TestFilters(string imgFN, string outDir)
{
	GS_ProblemInput probInput(imgFN);

	string fileID = Disk::GetFileNameID(Disk::GetBaseFileName(imgFN));	
	GS_Image fltImg;

	fltImg = GSF_Null::Null(probInput);
	ImageIO::WriteFile(fltImg, outDir + fileID + "-Null-default.tga");
}



