#include "GSF_Relight.h"
#include "ImageIO.h"
#include "Disk.h"


GS_Problem::Params GSF_Relight::GS_Problem_Params_Default()
{
	GS_Problem::Params params = GS_Problem::Params::Default();
	params.plsParams.levelCount = 1;
	params.channelsType = CT_YUV;
	return params;
}

GS_Solver::Params GSF_Relight::GS_Solver_Params_Default()
{
	GS_Solver::Params params = GS_Solver::Params::Default();	
	params.fidelity_data = 1.0e-2;
	params.channelsToSolve.insert(0);
	return params;
}


GS_Image GSF_Relight::Relight(GS_ProblemInput probInput,
							  Params fltParams,
							  GS_Problem::Params probParams,
							  GS_Solver::Params solParams)
{
	string paramsSpacing((TAB_SPACE_COUNT + 1) * TAB_LEVEL, ' ');
	
	REPORT("Creating GS problem... \n");
	probParams.Dump(paramsSpacing);
	GS_Problem prob(probInput, probParams);	

	REPORT("Filtering GS problem... \n");
	fltParams.Dump(paramsSpacing);
	GSF_Relight fltFunc(&prob, fltParams);
	if(solParams.levelsToSolve.size() == 0)
		fltFunc.Filter();
	else
		fltFunc.Filter(solParams.levelsToSolve);

	REPORT("Solving GS problem... \n");
	solParams.Dump(paramsSpacing);
	GS_Solver solver(&prob, solParams);
	solver.Solve();

	return prob.GetResultRGB();
}


void GSF_Relight::TestFilters(string imgFN, string outDir)
{
	GS_Problem::Params probParams           = GS_Problem_Params_Default();
	GS_ProblemInput::Params probInputParams = GS_ProblemInput::Params::Default();	
	GS_ProblemInput probInput(imgFN, probInputParams, probParams.channelsType);	

	string fileID = Disk::GetFileNameID(Disk::GetBaseFileName(imgFN));
	GS_Image fltImg;

	fltImg = GSF_Relight::Relight(probInput, Params::Default(), probParams);
	ImageIO::WriteFile(fltImg, outDir + "test-default-" + fileID + ".tga");
}

