#pragma once

#include "GS_Filter.h"
#include "Curve.h"
#include "Transform-2x2.h"

class GSF_Relight : public GS_Filter
{
public:

	typedef enum
	{
		LT_DIRC = 0,
		LT_POINT,
		LT_DEFAULT = LT_DIRC
	} LightType;

	typedef struct _Params 
	{
		float lightDircAng;
		float lightStrength;

		FVector2 lightPoint;
		bool     lightPointInv;

		LightType lightType;

		_Params(float _lightDirc = 45, float _lightStrength = 1.0f,
			    FVector2 _lightPoint = FVector2(0, 0), bool _lightPointInv = false,
				LightType _lightType = LT_DEFAULT)
		{
			this->lightDircAng  = _lightDirc;
			this->lightStrength = _lightStrength;
			this->lightPoint    = _lightPoint;    
			this->lightPointInv = _lightPointInv;
			this->lightType     = _lightType;
		}
		static _Params Default();
		void Dump(string prefix);
	}Params;

private:
	Params params;

public:
	GSF_Relight(GS_Problem *_problem, const Params &_params = Params::Default());
	void FilterLevel(int iLevel, void* userData = NULL);

	Params GetParams()
	{
		return this->params;
	}

private:
	string getScribblesFN(string orgFN);

public: //GSF_Relight-Driver.cpp	

	static GS_Problem::Params GS_Problem_Params_Default();
	static GS_Solver::Params GS_Solver_Params_Default();

	static GS_Image Relight(GS_ProblemInput probInput,
							Params fltParams              = Params::Default(),
							GS_Problem::Params probParams = GS_Problem_Params_Default(),
							GS_Solver::Params solParams   = GS_Solver_Params_Default());


	static void TestFilters(string imgFN, string outDir = "");
};


