#ifndef __GSF_SHARPEN_H__
#define __GSF_SHARPEN_H__

#include "GS_Filter.h"

class GSF_Sharpen : public GS_Filter
{
public:
	typedef struct _Params 
	{
		float multiplier;

		_Params(float _multiplier = 1.5f)
		{
			this->multiplier = _multiplier;
		}
		static _Params Default();
		static _Params Blur();
		static _Params Interpolate(const _Params &left, const _Params &right, float middleFrac);		
		void Dump(string prefix);
	}Params;

private:
	Params params;

public:
	GSF_Sharpen(GS_Problem* _problem, const Params &_params = Params::Default());
	void FilterLevel(int iLevel, void* userData = NULL);

	Params GetParams()
	{
		return this->params;
	}

public: //GSF_Sharpen-Driver.cpp
	static GS_Problem::Params GS_Problem_Params_Default();
	static GS_Problem::Params GS_Problem_Params_MultiFreq();
	static GS_Solver::Params  GS_Solver_Params_Default();	

	static GS_Image Sharpen(GS_Image inputImg,
							Params fltParams              = Params::Default(),
							GS_Problem::Params probParams = GS_Problem_Params_Default(),
							GS_Solver::Params solParams   = GS_Solver_Params_Default());

	static GS_Image Sharpen(GS_Problem* prob,
							Params fltParams              = Params::Default(),
							GS_Solver::Params solParams   = GS_Solver_Params_Default());


	static GS_Image SharpenFreq(GS_Image inputImg, int iFreqStart, int iFreqEnd,
								Params fltParams,
							    GS_Problem::Params probParams,
							    GS_Solver::Params solParams);

	static GS_Image SharpenHiFreq(GS_Image inputImg,
								  Params fltParams              = Params::Default(),
								  GS_Problem::Params probParams = GS_Problem_Params_MultiFreq(),
							      GS_Solver::Params solParams   = GS_Solver_Params_Default());

	static GS_Image SharpenMidFreq(GS_Image inputImg,
							       Params fltParams              = Params::Default(),
								   GS_Problem::Params probParams = GS_Problem_Params_MultiFreq(),
							       GS_Solver::Params solParams   = GS_Solver_Params_Default());

	static GS_Image SharpenLowFreq(GS_Image inputImg,
								   Params fltParams              = Params::Default(),
								   GS_Problem::Params probParams = GS_Problem_Params_MultiFreq(),
							       GS_Solver::Params solParams   = GS_Solver_Params_Default());

	static GS_Image Sharpen(GS_Image inputImg,
							Params startFltParams,
							Params endFltParams,
							GS_Problem::Params probParams = GS_Problem_Params_Default(),
							GS_Solver::Params solParams   = GS_Solver_Params_Default());

	static void TestFilters(string fileName, string outDir = "");

};

#endif

