#include "GSF_ToneMap.h"
#include "Disk.h"
#include "ImageIO.h"
#include "ColorHelper.h"

GS_Problem::Params GSF_ToneMap::GS_Problem_Params_Default()
{
	GS_Problem::Params params = GS_Problem::Params::Default();
	params.plsParams.levelCount = 1;

	params.channelsType = CT_LOG_LUM;

	GS_Problem::Filter filterColorize = GS_Problem::Filter::Default(PFT_LUM2RGB);
	params.postprocessFilters.push_back(filterColorize);

	GS_Problem::Filter filterRescale = GS_Problem::Filter::Default(PFT_ROBUST_RESCALE);
	params.postprocessFilters.push_back(filterRescale);

	//params.channelsType = CT_LOG_LUM;
	//GS_Problem::Filter filter = GS_Problem::Filter::Default(PFT_DENOISE);
	//filter.params.floatList[1] = 0.00003f;
	//params.preprocessFilters.push_back(filter);
	//params.channelsType = CT_LOG_RGB;
	return params;
}

GS_Solver::Params GSF_ToneMap::GS_Solver_Params_Default()
{
	GS_Solver::Params params = GS_Solver::Params::Default();
	
	params.fidelity_data  = 0.0;
	params.pinchFuncExp   = 3.0;
	params.pinchFuncScale = 1.0 / -log(LogLumE);

	params.accuracy = 1.0E-12;
	params.maxIters = 500;
	return params;
}

GS_Image GSF_ToneMap::ToneMap(GS_ProblemInput probInput,
							  Params fltParams,
							  GS_Problem::Params probParams,
							  GS_Solver::Params solParams)
{
	string paramsSpacing((TAB_SPACE_COUNT + 1) * TAB_LEVEL, ' ');
	
	REPORT("Creating GS problem... \n");
	probParams.Dump(paramsSpacing);
	GS_Problem prob(probInput, probParams);	

	REPORT("Filtering GS problem... \n");
	fltParams.Dump(paramsSpacing);
	GSF_ToneMap toneMapFlt(&prob, fltParams);
	if(solParams.levelsToSolve.size() == 0)
		toneMapFlt.Filter();
	else
		toneMapFlt.Filter(solParams.levelsToSolve);

	REPORT("Solving GS problem... \n");
	solParams.Dump(paramsSpacing);
	GS_Solver solver(&prob, solParams);
	solver.Solve();

	return prob.GetResultRGB();
}



void GSF_ToneMap::TestFilters(string imgFN, string outDir)
{
	GS_Problem::Params probParams = GS_Problem_Params_Default();
	GS_ProblemInput::Params probInputParams = GS_ProblemInput::Params::Default();
	GS_ProblemInput probInput(imgFN, probInputParams, probParams.channelsType);

	string fileID = Disk::GetFileNameID(Disk::GetBaseFileName(imgFN));
	GS_Image fltImg;
	Params defParams = Params::Default();
	
	fltImg = GSF_ToneMap::ToneMap(probInput, defParams, probParams);
	ImageIO::WriteFile(fltImg, outDir + "tb-default-" + fileID + ".tga");
}
