#ifndef __GF_INTEGRATOR_H__
#define __GF_INTEGRATOR_H__

#include "Definitions.h"
#include "SparseSqrSymMat.h"
#include "Image.h"

class GF_Integrator
{

public:
	typedef struct _GF_Integrator_Params
	{
		float  fidelity;
		int    maxIters;
		float  accuracy;
		static _GF_Integrator_Params Default()
		{
			_GF_Integrator_Params params;
			params.accuracy = 0.000000001f;
			params.maxIters = 50;
			params.fidelity = 0.001f;
			return params;
		}

		void Dump()
		{
			printf("GF Integrator Params:\n");
			printf("\t fidelity: %f\n", fidelity);
			printf("\t maxIters: %i\n", maxIters);
			printf("\n");
		}
	} GF_Integrator_Params;


private:
	CFloatImage orgImg;
	CFloatImage solImgGx;
	CFloatImage solImgGy;

	GF_Integrator::GF_Integrator_Params params;
	
public:
	GF_Integrator(CFloatImage _orgImg, 
				  CFloatImage _solImgGx,
				  CFloatImage _solImgGy,
				  GF_Integrator::GF_Integrator_Params _params);

	void Solve(CFloatImage &solImg);

private:
	void integrateChannel(int iChannel, CFloatImage &solImg);
	void defineChannelEqs(int iChannel, SparseSqrSymMat *matA, precise *vecX, precise *vecB);
};

#endif