#ifndef __CG_SOLVER_H__
#define __CG_SOLVER_H__

#include "Definitions.h"
#include "LinearSolverDefs.h"
#include "AbstractMat.h"
#include "AbsractPreconditioner.h"
#include "Vec.h"

class LinearSolver
{

public:
	typedef struct _LinearSolverParams
	{
		precise errTolerance;
		int maxIters;
		int fltErrMaxiters;

		static _LinearSolverParams Default();
	}LinearSolverParams;

	//Solve linear system using gradient descent
	static void Solve_GD(int n, AbstractMat *A, precise *x, precise *b, 
						 LinearSolverParams params = LinearSolverParams::Default());

	//Solve linear system using conjugate gradients
	static void Solve_CG(int n, AbstractMat *A, precise *x, precise *b, 
						 LinearSolverParams params = LinearSolverParams::Default());

	//Solve linear system using preconditioned conjugate gradients
	static void Solve_PCG(int n, AbstractMat *A, precise *x, precise *b, 
					 	  LinearSolverParams params = LinearSolverParams::Default(),
						  AbsractPreconditioner* MInv = &NoopPreconditioner());


	static precise GetResidual(int n, AbstractMat *A, precise *x, precise *b);

public:
	static void SolveRandSys_GD(int size);
};


#endif
