#include "SparseSqrSymMat.h"

void SparseSqrSymMat::Dump()
{
	printf("Matrix (%i x %i):\n", this->n, this->n);

	for(int iRow = 0; iRow < this->n; iRow++)
	for(int iCol = 0; iCol < this->n; iCol++)
	{		
		if(iCol != this->n - 1)
			printf("%0.2f, ", GetElem(iRow, iCol));
		else
			printf("%0.2f\n", GetElem(iRow, iCol));
	}
}


void SparseSqrSymMat::DumpMatlab(char* matName)
{
	printf("%s = [ ", matName, this->n, this->n);

	for(int iRow = 0; iRow < this->n; iRow++)
	for(int iCol = 0; iCol < this->n; iCol++)
	{		
		if(iCol != this->n - 1)
			printf("%1.4e ",  GetElem(iRow, iCol));
		else
			printf("%1.4e; ", GetElem(iRow, iCol));
	}

	printf("]\n");
}


void SparseSqrSymMat::DumpRandVecMult(int size)
{
	SparseSqrSymMat mat;
	precise *x = new precise[size];
	precise *b = new precise[size];
	INSIST(x != NULL);
	INSIST(b != NULL);

	mat = GenRandMat(size);
	Vec::FillRandValues(size, x);
	mat.Multiply(size, size, x, b);

	mat.DumpMatlab();	
	Vec::DumpMatlab(size, x);
	Vec::DumpMatlab(size, b);

	delete [] x;
	delete [] b;
}


SparseSqrSymMat SparseSqrSymMat::GenRandMat(int size)
{
	SparseSqrSymMat mat(size);

	for(int i = 0; i < size; i++)
	{
		int rowElemCount = myRand() % (size + 1);
		for(int j = 0; j < rowElemCount; j++)
		{
			int iCol = myRand() % size;
			if(i <= iCol)
				mat.SetElem(i, iCol, genProb());
		}
	}

	return mat;
}


SparseSqrSymMat SparseSqrSymMat::GenIdenMat(int size)
{
	SparseSqrSymMat mat(size);

	for(int i = 0; i < size; i++)
	{
		mat.SetElem(i, i, 1);
	}

	return mat;
}