#ifndef __SPARSE_MAT_H__
#define __SPARSE_MAT_H__

#include "Definitions.h"
#include "LinearSolverDefs.h"
#include "AbstractMat.h"
#include "Vec.h"
#include <map>

class SparseSqrSymMat : public AbstractMat
{
	typedef std::map<int, precise> MatRow;
	vector<MatRow> matData; 
	int n;

public:
	SparseSqrSymMat(int _n = 0);
	~SparseSqrSymMat();

	void SetElem(int iRow, int iCol, precise val)
	{
		if(val == 0) return;

		INSIST(iRow <= iCol);
		CHECK_RANGE(iRow, 0, this->n - 1);
		CHECK_RANGE(iCol, 0, this->n - 1);

		this->matData[iRow][iCol] = val;
	}

	precise GetElem(int iRow, int iCol) const
	{
		CHECK_RANGE(iRow, 0, this->n - 1);
		CHECK_RANGE(iCol, 0, this->n - 1);
		if(iRow > iCol)
		{
			int temp = iRow;
			iRow = iCol;
			iCol = temp;
		}

		MatRow::const_iterator findResult = this->matData[iRow].find(iCol);
		if(findResult == this->matData[iRow].end())
			return 0;
		else
			return findResult->second;
	}

	int GetRowCount() const
	{
		return this->n;
	}

	int GetColCount() const
	{
		return this->n;
	}

	void Multiply(int inLen, int outLen, 
				  const precise *inVec, precise *outVec) const;

public: //SparseSqrSymMat-Debug.cpp
	void Dump();
	void DumpMatlab(char* matName = "A");
	static void DumpRandVecMult(int size);
	static SparseSqrSymMat GenRandMat(int size);
	static SparseSqrSymMat GenIdenMat(int size);
};

#endif