#ifndef __DEFINITIONS_H__
#define __DEFINITIONS_H__

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include <limits.h>
#ifdef WIN32
#pragma warning(disable: 4702)
#pragma warning(disable: 4204)
#endif
#include <vector>
using namespace std;
#include "Debug.h"
#include <malloc.h>
#include <set>
#include "half.h"

extern bool GlobalDebugVar;


#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif
#define M_PI_FLOAT 3.14159265358979323846f

#ifndef uchar 
typedef unsigned char uchar;
#endif

const uchar MASK_VALID   = UCHAR_MAX;
const uchar MASK_INVALID = 0;

#ifndef uint
typedef unsigned int uint;
#endif

#ifndef strings
typedef vector<string> strings;
#endif

#ifndef ints
typedef vector<int> ints;
#endif

#ifndef floats
typedef vector<float> floats;
#endif

#ifndef SortedInts
typedef set<int> SortedInts;
#endif


#ifndef WeightIndex
typedef std::pair<float, int> WeightIndex;
#endif

#define SQRT_2 1.41421356237309504880f

typedef std::pair<int, int> IntPair;

#define SET_IF_LT(var, val) if(val < var) var = val;

#define SET_IF_GT(var, val) if(val > var) var = val;

#define SQR(x) ((x) * (x))

typedef void (*CallbackFunc)(void*);
typedef std::pair<void*, CallbackFunc> Callback;


typedef double (*CallbackFunc_Double)(const void*, double);
typedef short (*CallbackFunc_Short)(const void*, short);

typedef std::pair<const void*, CallbackFunc_Double> Callback_Double;
typedef std::pair<const void*, CallbackFunc_Short> Callback_Short;

inline uint NEAREST_UINT(float n)
{
	ENSURE(n >= 0.0f);
	return (uint) (n + 0.5f);
}

inline int NEAREST_INT(float n)
{
	return (int) (n + 0.5f);
}

inline int NEAREST_INT(double n)
{
	return (int) (n + 0.5);
}

template <class T>
inline bool InRange(T x, T min, T max)
{
	return ((x >= min) && (x <= max)) ? true : false;
}

#ifdef WIN32
	#include <direct.h>
	#include <typeinfo.h>
	#include <io.h>

	const int MY_RAND_MAX = ((RAND_MAX << 15) + RAND_MAX);
	inline int myRand()
	{
		return ((rand() << 15) + rand());
	}

	#include <hash_set>
	#include <hash_map>
	using namespace stdext;

	#define COPY_CMD   "copy"
	#define RENAME_CMD "ren"	

	const char   OS_PATH_DELIMIT     = '\\';
	const string OS_PATH_DELIMIT_STR = "\\";
	const char WRONG_OS_PATH_DELIMIT = '/';

	#define MKDIR(dirName, permissions) mkdir(dirName)

#else

	#include <sys/io.h>
	#include <sys/stat.h>
	#include <sys/types.h>
	#include <unistd.h>
	#include <typeinfo>
	#include <glob.h>
	#include <dirent.h>

    const int MY_RAND_MAX = RAND_MAX;
	inline int myRand()
	{
		return rand();
	}

	#include <ext/hash_set>
	#include <ext/hash_map>
	using namespace __gnu_cxx;
	namespace __gnu_cxx
	{
		template<> struct hash< std::string >
		{
			size_t operator()( const std::string& x ) const
			{
				return hash< const char* >()( x.c_str() );
			}
		};
	}

	#define COPY_CMD   "cp"
	#define RENAME_CMD "mv"	

	const char OS_PATH_DELIMIT       = '/';
	const string OS_PATH_DELIMIT_STR = "/";
	const char WRONG_OS_PATH_DELIMIT = '\\';

	#define MKDIR(dirName, permissions) mkdir(dirName, permissions)

#endif

template <class T>
T myAbs(T input)
{
	return (input < 0) ? -input : input;
}

inline float genProb()
{
	return ((float) myRand() / MY_RAND_MAX);
}

void Tokenize(const string& str,
              vector<string>& tokens,
              const string& delimiters = " \t");

bool SilentSysCall(const char *cmd, 
				   bool silenceStdOut = true, 
				   bool silenceStdErr = true);

string tolower(string str);

bool beginsWith(string str, string substr);
string trimStart(string str, size_t count);


#endif


