#include "LineIntersection.h"
#include "matrix.h"

FVector2 LineIntersection::GetIntersection(vector<LineEq> &lineEquations)
{
	FVector2 intersectionPt;

	int lineCount = (int) lineEquations.size();
	INSIST(lineCount >= 2);

	const int n = 2;
	int m = lineCount;

	double *A, *b; 
	double x[n];

	A = new double[m * n];
	b = new double[m];
	INSIST(A != NULL);
	INSIST(b != NULL);

	uint aAddr = 0;
	for(int iLine = 0; iLine < lineCount; iLine++, aAddr += n)
	{
		A[aAddr + 0] =  lineEquations[iLine].a;
		A[aAddr + 1] =  lineEquations[iLine].b;
		b[iLine]     = -lineEquations[iLine].c;
	}

	dgelsy_driver(A, b, x, m, n, 1);
	intersectionPt[0] = (float) x[0];
	intersectionPt[1] = (float) x[1];

	//check for parallel lines
	if((x[0] != x[0]) || (x[0] > DBL_MAX) ||
	   (x[1] != x[1]) || (x[1] > DBL_MAX))
	{
		//bug
		REPORT_FAILURE("Parallel lines not handled");
	}


	delete [] A;
	delete [] b;

	return intersectionPt;
}