#include "Curve.h"

void Curve::Load(string curveStr)
{
	vector<string> curveOpts;
	Tokenize(curveStr, curveOpts, " \t");
	Load(curveOpts);
}

void Curve::Load(vector<string> &optVals)
{	
	const int headerParamCount = 4;
	INSIST(optVals.size() >= (size_t) headerParamCount);

	int matched;	

	int _funcSamples;
	matched = sscanf(optVals[0].c_str(), "%i", &_funcSamples);
	INSIST(matched == 1);
	INSIST(_funcSamples > 0);

	matched = sscanf(optVals[1].c_str(), "%lf", &this->minOut);
	INSIST(matched == 1);

	matched = sscanf(optVals[2].c_str(), "%lf", &this->maxOut);
	INSIST(matched == 1);

	int pointCount;
	matched = sscanf(optVals[3].c_str(), "%i", &pointCount);
	INSIST(matched == 1);

	INSIST(pointCount >= 2);
	INSIST((int) optVals.size() == (pointCount * 2) + headerParamCount);
	this->points.clear();
	for(int iPoint = 0; iPoint < pointCount; iPoint++)
	{
		CurvePoint pt;

		matched = sscanf(optVals[headerParamCount + (2 * iPoint) + 0].c_str(), "%lf", &pt.x);
		INSIST(matched == 1);

		matched = sscanf(optVals[headerParamCount + (2 * iPoint) + 1].c_str(), "%lf", &pt.y);
		INSIST(matched == 1);

		insert(pt, _funcSamples);
	}	
}

string Curve::ToString()
{
	string objStr;

	char buf[1024];
	string bufStr;

	sprintf(buf, "%i", this->funcSamples);
	bufStr = buf;
	objStr += bufStr  + " ";

	sprintf(buf, "%e", this->minOut);
	bufStr = buf;
	objStr += bufStr  + " ";

	sprintf(buf, "%e", this->maxOut);
	bufStr = buf;
	objStr += bufStr  + " ";

	sprintf(buf, "%i", (int) this->points.size());
	bufStr = buf;
	objStr += bufStr  + " ";

	for(uint iPoint = 0; iPoint < this->points.size(); iPoint++)
	{
		sprintf(buf, "%e %e", this->points[iPoint].x, this->points[iPoint].y);
		bufStr = buf;
		objStr += bufStr  + "  ";
	}

	return objStr;
}


void Curve::Dump(string prefix)
{
	const char* prefixPtr = prefix.c_str();

	printf("%s  funcSamples      - %i\n", prefixPtr, this->funcSamples);
	printf("%s  (minOut, maxOut) - (%0.03f, %0.03f)\n", prefixPtr, this->minOut, this->maxOut);
	printf("%s  Points:\n", prefixPtr);
	for(uint iPoint = 0; iPoint < this->points.size(); iPoint++)
	{
		printf("%s\t [%0.05f, %0.05f]\n", prefixPtr, this->points[iPoint].x, this->points[iPoint].y);
	}

	printf("\n");
}


