#ifndef __CURVE_POINTS_H__
#define __CURVE_POINTS_H__

#include "Definitions.h"

typedef struct _CurvePoint
{
	double x, y;
	_CurvePoint(double _x = 0.0, double _y = 0.0) 
	{
		this->x = _x; this->y = _y;
	}
} CurvePoint;

class Curve
{
private:
	vector<CurvePoint> points;
	double minOut, maxOut;
	double* funcMap;
	int funcSamples;

public:
	Curve();
	~Curve();
	Curve& operator= (const Curve& srcCurve);
	Curve(const Curve& srcCurve);
	
	double MinOut() const
	{
		return this->minOut;
	}

	double MaxOut() const
	{
		return this->maxOut;
	}	

	int size() const
	{
		return (int) this->points.size();
	}

	CurvePoint at(int index) const
	{
		return points[index];
	}

	const CurvePoint operator[](int index) const
	{
		CurvePoint p = points[index];
		return p;
	}	

	int insert(CurvePoint curvePoint, int _funcSamples = 500);

	void modify(int index, CurvePoint curvePoint);

	void erase(int index);

	void Dump();

	int GetNearestPointIndex(double x, double y, double dist_thres);

	static double GetCurveResponse(const void *curvePtr, double x);
	double GetCurveResponse(double x) const;
	double GetCurveResponseFast(double x) const;

	void MinOut(double _minOut);
	void MaxOut(double _maxOut);

private:
	void adjustResponseRange(double newMin, double newMax);
	
public: //Curve-CatRom.cpp
	void ComputeFuncMap(int _funcSamples = 500);

private: //Curve-CatRom.cpp

	void computeFuncMapFast(int _funcSamples);
	void computeFuncMapPrecise(int _funcSamples);

	void computeSegFuncMap(const CurvePoint &p1, const CurvePoint &p2, 
						   const CurvePoint &p3, const CurvePoint &p4,
						   int segSamples);

	void computeFuncMapHoles();	

	double GetCurveResponsePrecise(double x, double xDistThres = 0.001) const;

	double getCurveResponsePrecise(const CurvePoint &p1, const CurvePoint &p2, 
			         			   const CurvePoint &p3, const CurvePoint &p4,
						           double x, double xDistThres) const;

public: //Curve-IO.cpp	
	void Load(vector<string> &optVals);	
	void Load(string curveStr);	
	string ToString();
	void Dump(string prefix);
};

#endif

