#include "GS_MainWindow.h"
#include "ImageIO.h"
#include <FL/Fl_File_Icon.H>
#include "Disk.h"

void GS_MainWindow::setup_Widgets()
{	
	this->begin();

	int widgetLocX = 0, widgetLocY = 0;

	setupMenubar(widgetLocX, widgetLocY);
	widgetLocY += this->wg_menubar->y() + this->wg_menubar->h();

	setupImgPanel(widgetLocX, widgetLocY);
	widgetLocY += this->wg_imgPanel->y() + this->wg_imgPanel->h();

	setupFileChooser();	
	setupFilterWindow();
	resetViewCheck();

	this->end();
}

void GS_MainWindow::setupMenubar(int xLoc, int yLoc)
{
	this->wg_menubar = new Fl_Menu_Bar(xLoc, yLoc, this->w(), 30, "wg_menubar");
	setupMenubarFile();
}

void GS_MainWindow::setupMenubarFile()
{
	this->wg_menubar->add("&File/&Open...",           0, (Fl_Callback *) cb_file_open,         this);
	this->wg_menubar->add("&File/Save result &as...", 0, (Fl_Callback *) cb_file_saveResultAs, this);
	this->wg_menubar->add("&File/&Exit",              0, (Fl_Callback *) cb_file_exit,         this);

	this->wg_menubar->add("&View/&Input img",  0, (Fl_Callback *) cb_view_img,    this, FL_MENU_TOGGLE);
	this->wg_menubar->add("&View/&Result img", 0, (Fl_Callback *) cb_view_result, this, FL_MENU_TOGGLE);

	this->wg_menubar->add("&Window/&filter",   0, (Fl_Callback *) cb_win_filter,  this, FL_MENU_TOGGLE);
}

void GS_MainWindow::setupFileChooser()
{
	int extCount = sizeof(IMAGE_IO_EXT_STRS) / sizeof(*IMAGE_IO_EXT_STRS);
	INSIST(extCount > 0);

	vector<string> fileExtStrs,	fileExtType;
	for(int iExt = 0; iExt  < extCount; iExt++)
	{
		string currExtStrs = IMAGE_IO_EXT_STRS[iExt];
		string currExtType = IMAGE_IO_EXT_TYPE[iExt];

		fileExtStrs.push_back(currExtStrs);
		fileExtType.push_back(currExtType);
	}

	this->wg_fileChooser = new GS_FileChooser(fileExtStrs, fileExtType);
}



void GS_MainWindow::setupImgPanel(int xLoc, int yLoc)
{
	this->wg_imgPanel = new GS_ImagePanel(&this->imgData);
	this->wg_imgPanel->position(xLoc, yLoc);

	Callback resize_callback(this, &GS_MainWindow::updateSize);
	this->wg_imgPanel->Add_resize_callback(resize_callback);
}

void 
GS_MainWindow::setupFilterWindow()
{
	this->wg_filterWindow = new GS_FilterWindow(this, &this->params.options.fltParams);
}


//called when one of the child widget changes its size
void GS_MainWindow::updateSize(void* mainWindowPtr)
{
	GS_MainWindow *mainWin = (GS_MainWindow *) mainWindowPtr;
	mainWin->updateSize();
}

void GS_MainWindow::updateSize()
{
	int newWidth  = 0;
	int newHeight = 0;
	
	SET_IF_GT(newWidth, this->wg_imgPanel->x() + this->wg_imgPanel->w());
	SET_IF_GT(newHeight, this->wg_imgPanel->y() + this->wg_imgPanel->h());

	//update menubar width to match the width of the main-window
	this->wg_menubar->size(newWidth - this->wg_menubar->x(), 
						   this->wg_menubar->h());

	this->size(newWidth, newHeight);
}


void 
GS_MainWindow::resetViewCheck()
{
	int viewMenuIndex = getIndexByName(this->wg_menubar, "&View");
	INSIST(viewMenuIndex != -1);

	Fl_Menu_Item* viewItem = (Fl_Menu_Item*)&(this->wg_menubar->menu()[viewMenuIndex]);
	Fl_Menu_Item* winItem = getMenuItemByName(this->wg_menubar, "&Window");
	INSIST(viewItem != NULL);
	INSIST(winItem != NULL);
	if(this->imgData.imgShape.NodeCount() == 0)
	{
		viewItem->deactivate();
		winItem->deactivate();
	}
	else
	{
		viewItem->activate();
		winItem->activate();
	}
	
	int viewMenuItemIndex = viewMenuIndex + 1;
	while(true)
	{
		Fl_Menu_Item* viewMenuItem = (Fl_Menu_Item*)&(this->wg_menubar->menu()[viewMenuItemIndex]);
		INSIST(viewMenuItem->submenu() == false);

		if(viewMenuItem->label() == NULL)
			break;
		else
			viewMenuItem->clear();

		viewMenuItemIndex++;
	};

	int panelType = (int) this->wg_imgPanel->GetPanelType();
	CHECK_RANGE(panelType, 0, viewMenuItemIndex - viewMenuIndex);


	Fl_Menu_Item* selectedItem = (Fl_Menu_Item*)&(this->wg_menubar->menu()[viewMenuIndex + panelType + 1]);
	selectedItem->set();
}
