#include "GS_MainWindow.h"
#include "Timer.h"

void 
GS_MainWindow::setupFilters()
{
	delete this->gsf_toneMap;
	delete this->gsf_sharpen;
	delete this->gsf_painterly;
	delete this->gsf_ele;
	delete this->gsf_deblock;
	delete this->gsf_relight;
	delete this->gss_fourier;

	this->gsf_toneMap   = new GSF_ToneMap(this->imgData.prob, 
										  this->params.options.fltParams.gsf_tmParams);
	this->gsf_sharpen   = new GSF_Sharpen(this->imgData.prob, 
						 				  this->params.options.fltParams.gsf_SharpenParams);
	this->gsf_painterly = new GSF_NPR(    this->imgData.prob, 
									      this->params.options.fltParams.gsf_nprParams);
	this->gsf_ele       = new GSF_ELE(    this->imgData.prob, 
										  this->params.options.fltParams.gsf_EleParams);
	this->gsf_deblock   = new GSF_Deblock(this->imgData.prob, 
										  this->params.options.fltParams.gsf_deblockParams);

	this->gsf_relight   = new GSF_Relight(   this->imgData.prob, 
											 this->params.options.fltParams.gsf_relightParams);
	
	this->gss_fourier = new GS_FourierSolver(this->imgData.prob,
											 this->params.options.fltParams.gssParams);

	INSIST(this->gsf_toneMap   != NULL);
	INSIST(this->gsf_sharpen   != NULL);
	INSIST(this->gsf_painterly != NULL);
	INSIST(this->gsf_ele       != NULL);
	INSIST(this->gsf_deblock   != NULL);
	INSIST(this->gsf_relight   != NULL);
	INSIST(this->gss_fourier   != NULL);
}


void 
GS_MainWindow::executeFilter(Filter_Driver::FT_Type filterType)
{
	Timer timer;

	printf("Applying filter...\n");
	switch(filterType)
	{
	case Filter_Driver::FT_SHARPEN:
		this->gsf_sharpen->Filter(&this->params.options.fltParams.gsf_SharpenParams);
		break;

	case Filter_Driver::FT_TM:
		this->gsf_toneMap->Filter(&this->params.options.fltParams.gsf_tmParams);
		break;

	case Filter_Driver::FT_ELE:
		this->gsf_ele->Filter(&this->params.options.fltParams.gsf_EleParams);
		break;

	case Filter_Driver::FT_NPR:
		this->gsf_painterly->Filter(&this->params.options.fltParams.gsf_nprParams);
		break;

	case Filter_Driver::FT_RELIGHT:
		this->gsf_relight->Filter(&this->params.options.fltParams.gsf_relightParams);
		break;

	case Filter_Driver::FT_DEBLOCK:
		this->gsf_deblock->Filter(&this->params.options.fltParams.gsf_deblockParams);
		break;

	default:
		REPORT_FAILURE("Unknown filter type - %i\n", filterType);
		break;
	};
	timer.DumpSeconds("\tDone - ");

	executeSolver();
}

void 
GS_MainWindow::executeSolver()
{
	this->gss_fourier->SetParams(this->params.options.fltParams.gssParams);
	this->gss_fourier->Solve();
	view_result();
}



