#ifndef __GS_MAIN_WINDOW_H__
#define __GS_MAIN_WINDOW_H__

#include "Definitions.h"
#include "GS_UI_Definitions.h"
#include "ImageData.h"
#include "GS_FileChooser.h"
#include "GS_ImagePanel.h"
#include "GS_Curve_Panel.h"
#include "OptFileParser.h"
#include "GS_FilterWindow.h"

class GS_MainWindow : public Fl_Double_Window
{
public:
	//definitions
	typedef struct _GS_MainWindowParams
	{
		int width, height;
		string title;
		OptFileParser::OptFileParams options;

		static _GS_MainWindowParams Default();
	} GS_MainWindowParams;

private: 
	//variables	
	Fl_Menu_Bar*     wg_menubar; 
	GS_FileChooser*  wg_fileChooser;
	GS_ImagePanel*   wg_imgPanel;
	GS_FilterWindow* wg_filterWindow;

	GS_MainWindowParams params;
	ImageData imgData;

	GSF_ToneMap*      gsf_toneMap;
	GSF_Sharpen*      gsf_sharpen;
	GSF_NPR*          gsf_painterly;
	GSF_ELE*          gsf_ele;
	GSF_Deblock*      gsf_deblock;
	GSF_Relight*      gsf_relight;


	GS_FourierSolver* gss_fourier;

//Funtions

public: //GS_MainWindow.cpp
	GS_MainWindow(GS_MainWindowParams params = GS_MainWindowParams::Default());
	void setup_VarDefaults();
	
private: //GS_MainWindow-WG.cpp
	void setup_Widgets();
	void setupMenubar(int xLoc, int yLoc);
	static void updateSize(void* mainWindowPtr);
	void updateSize();
	void setupMenubarFile();
	void setupFileChooser();
	void setupFilterWindow();
	void setupImgPanel(int xLoc, int yLoc);
	void resetViewCheck();

private: //GS_MainWindow-CB-File.cpp	
	void file_open();
	void file_open(string fileName);
	void file_saveResult(string fileName);
	void file_saveResultAs();
	void file_exit();

	static void cb_file_open(Fl_Widget*, void* mainWinPtr);
	static void cb_file_saveResultAs(Fl_Widget*, void* mainWinPtr);
	static void cb_file_exit(Fl_Widget*, void* mainWinPtr);

private: //GS_MainWindow-CB-View.cpp	
	void view_img();
	void view_result();

	static void cb_view_img(Fl_Widget*, void* mainWinPtr);
	static void cb_view_result(Fl_Widget*, void* mainWinPtr);

public: //GS_MainWindow-CB-Filter.cpp
	void win_filter();
	void win_filter_closed();
	static void cb_win_filter(Fl_Widget*, void* mainWinPtr);

public: //GS_MainWindow-Events.cpp
	int handle(int eventID);

public: //GS_MainWindow-menuHelper.cpp
	int getIndexByName(Fl_Menu_Bar* menubar, const char *findname);
	Fl_Menu_Item* getMenuItemByName(Fl_Menu_Bar* menubar, const char *findname);
	void setRadioByName(Fl_Menu_Bar *menubar, const char *menuname);
	void setToggleByName(Fl_Menu_Bar *menubar, const char *menuname, int val );
	void setActivateByName(Fl_Menu_Bar *menubar, const char *menuname, int val);
	int getToggleValueByName(Fl_Menu_Bar *menubar, const char *menuname);
	void clearItemsBelow(Fl_Menu_Bar *menubar, const char *menuname);


public:
	void setupFilters();
	void executeFilter(Filter_Driver::FT_Type);
	void executeSolver();
};


#endif