#include "GS_FileChooser.h"
#include "Disk.h"

GS_FileChooser::GS_FileChooser(vector<string> fileExtStrs, 
							   vector<string> fileExtType) : Fl_File_Chooser("", "", 0, "")
{
	Fl_File_Icon::load_system_icons();

	this->extStrs  = fileExtStrs;
	this->extType  = fileExtType;
	this->extCount = (int) this->extStrs.size();
	INSIST(this->extCount == this->extType.size());

	//default settings
	string fileFilter  = getFilterStr();
	string startDir    = "";
	string title       = "Choose file";
	int    showPreview = 0;
	int    fcType      = Fl_File_Chooser::SINGLE;

	this->filter(fileFilter.c_str());
	this->directory(startDir.c_str());
	this->preview(showPreview);
	this->label(title.c_str());
	this->type(fcType);
}

string GS_FileChooser::GetFileNameChoice(string winTitle, 
										 int fileChooserType, 
										 string defaultExt, 
										 bool warnIfFileExists)
{
	this->type( fileChooserType );
	this->label( winTitle.c_str() );
	setFilterIndex(defaultExt);
	this->show();	

	while(this->visible())
	{
		Fl::wait();
	}

	string fileName = "";

	if(this->count() > 0)
	{
		INSIST(this->count() == 1);
		INSIST(this->value(1) != NULL);

		char fileNameBuf[1024];
		fl_filename_relative(fileNameBuf, sizeof(fileNameBuf), this->value(1));
		fileName = fileNameBuf;

		if(fileName.length() > 0)
		{
			fileName = addFileExtIfMissing(fileName);
			if(warnIfFileExists && Disk::DoesFileExist(fileName))
			{
				char warning[1024];
				sprintf(warning, "Replace file: %s", fileName.c_str());
				if(fl_choice(warning, "No", "Yes", NULL) == 0)
				{
					fileName = "";
				}
			}
		}
	}

	return fileName;
}


string GS_FileChooser::getFilterStr()
{	
	string fileFilter = "";
	for(int iExt = 0; iExt < this->extCount; iExt++)
	{
		string ext     = this->extStrs[iExt];
		string extType = this->extType[iExt];		
		
		if(iExt != 0)
		{
			fileFilter += "\t";
		}

		fileFilter +=  extType + " (*.{" + ext + "})";		
	}

	return fileFilter;
}

void GS_FileChooser::setFilterIndex(string imgExt)
{
	int defaultExtIndex = max(0, this->extCount - 1);
	int extIndex        = defaultExtIndex;
	if(imgExt.length() > 0)
	{
		imgExt = tolower(imgExt);
	
		for(int iExtStrs = 0; iExtStrs < this->extCount; iExtStrs++)
		{
			string currExtStrs = this->extStrs[iExtStrs];
			vector<string> exts;
			Tokenize(currExtStrs, exts, ","); 
			INSIST(exts.size() > 0);

			for(uint iExt = 0; iExt < exts.size(); iExt++)
			{
				if(imgExt.compare(exts[iExt]) == 0)
				{
					extIndex = iExtStrs;
					break;
				}
			}

			if(extIndex != defaultExtIndex)
			{
				break;
			}
		}
	}
	
	this->filter_value(extIndex); 
}

string GS_FileChooser::addFileExtIfMissing(string fileName)
{
	string fileNameWithExt = fileName;

	string fileExt = Disk::GetFileNameExt(Disk::GetBaseFileName(fileName));	
	if((fileExt.length() == 0) &&
	   (this->extCount > 0))
	{
		int extFilterIndex = this->filter_value();
		
		if(extFilterIndex >= this->extCount)
		{
			extFilterIndex = 0;
		}

		string currExtStrs = this->extStrs[extFilterIndex];
		vector<string> exts;
		Tokenize(currExtStrs, exts, ","); 
		INSIST(exts.size() > 0);

		fileExt = exts[0];

		if(fileName[fileName.length() - 1] != '.')
			fileName += ".";
		fileNameWithExt = fileName + fileExt; 
	}

	return fileNameWithExt;
}
