#include "GS_Curve_Panel.h"
#include <FL/fl_draw.H>

GS_Curve_Panel::GS_Curve_Panel(Curve *_curvePoints, GS_Curve_Panel_Params _params) : Fl_Group(0,0,0,0)
{
	this->params = _params;	
	this->curvePoints = _curvePoints;

	setup_VarDefaults();
	setup_Widgets();
	paramUpdate();

	resize(this->params.width, this->params.height);
}

GS_Curve_Panel::~GS_Curve_Panel()
{
	delete this->wg_minR_input;
	delete this->wg_maxR_input;
	delete this->wg_in_input;
	delete this->wg_out_input;
	delete this->wg_Curve_Vis;
}

GS_Line GS_Curve_Panel::GetGraphDiag()
{
	return this->wg_Curve_Vis->GetGraphDiag();
}

void GS_Curve_Panel::setup_VarDefaults()
{
	this->wg_minR_input = NULL;
	this->wg_maxR_input = NULL;
	this->wg_in_input   = NULL;
	this->wg_out_input  = NULL;
	this->wg_Curve_Vis  = NULL;

	this->selectedPtIndex = -1;
}

void GS_Curve_Panel::setup_Widgets()
{
	this->box(FL_PLASTIC_DOWN_BOX);

	this->begin();
	setupCurvePoints();
	setupCurveVis();	
	setupResponseInputs();	
	setupInOutInputs();
	this->end();
}

void GS_Curve_Panel::setupCurvePoints()
{
	this->selectedPtIndex = -1;
}

void GS_Curve_Panel::setupInOutInputs()
{	
	this->wg_in_input= new Fl_Float_Input(0,0,0,0, "in:");	
	this->wg_in_input->when(FL_WHEN_ENTER_KEY);
	this->wg_in_input->callback(&GS_Curve_Panel::InValChanged, this);

	this->wg_out_input = new Fl_Float_Input(0,0,0,0, "out:"); 	
	this->wg_out_input->when(FL_WHEN_ENTER_KEY);
	this->wg_out_input->callback(&GS_Curve_Panel::OutValChanged, this);

	updateInOutInputs();
}

void GS_Curve_Panel::setupResponseInputs()
{
	this->wg_minR_input = new Fl_Float_Input(0,0,0,0,NULL);
	this->wg_minR_input->when(FL_WHEN_ENTER_KEY);
	this->wg_minR_input->callback(&GS_Curve_Panel::MinResponseChanged, this);

	this->wg_maxR_input = new Fl_Float_Input(0,0,0,0,NULL); 
	this->wg_maxR_input->when(FL_WHEN_ENTER_KEY);
	this->wg_maxR_input->callback(&GS_Curve_Panel::MaxResponseChanged, this);

	updateResponseInputs();
}

void GS_Curve_Panel::setupCurveVis()
{
	this->wg_Curve_Vis = new GS_Curve_Vis(this->curvePoints);
}

void GS_Curve_Panel::resize(int w, int h)
{
	this->size(w, h);

	int xPadding = 5;
	int yPadding = 5;

	int inputSizeY      = 20;
	int inputLabelWidth = 40;

	/*
	 * setup int and out response input-fields
	 */
	int in_X = xPadding + inputLabelWidth;
	int in_Y = yPadding;
	int in_W = (int) (this->w() - 
	                  (3 * xPadding) - 
					  (inputLabelWidth + inputLabelWidth)) / 2;
	int in_H = inputSizeY;
	this->wg_in_input->resize(in_X, in_Y, in_W, in_H);

	int out_X = in_X + in_W + xPadding + inputLabelWidth;
	int out_Y = in_Y;
	int out_W = in_W;
	int out_H = in_H;
	this->wg_out_input->resize(out_X, out_Y, out_W, out_H);

	/*
	 * setup curve graph visualizer
	 */
	int visX = xPadding;
	int visY = in_Y + in_H + yPadding;
	int visW = this->w() - (2 * xPadding);
	int visH = (this->y() + this->h()) - visY - yPadding;
	this->wg_Curve_Vis->resize(visX, visY, visW, visH);


	GS_Line graphDiag = this->wg_Curve_Vis->GetGraphDiag();

	int maxR_X = xPadding;
	int maxR_Y = graphDiag.y2;
	int maxR_W = 26;
	int maxR_H = 20;
	this->wg_maxR_input->resize(maxR_X, maxR_Y, maxR_W, maxR_H);

	int minR_X = maxR_X;
	int minR_W = maxR_W;
	int minR_H = maxR_H;
	int minR_Y = graphDiag.y1  - (minR_H / 2);
	this->wg_minR_input->resize(minR_X, minR_Y, minR_W, minR_H);
}

void GS_Curve_Panel::paramUpdate()
{
	this->wg_Curve_Vis->UpdateParams(this->selectedPtIndex);
	updateInOutInputs();
}

