#ifndef __GS_Curve_Panel_H__
#define __GS_Curve_Panel_H__

#include "Definitions.h"
#include "GS_UI_Definitions.h"
#include "GS_Curve_Vis.h"
#include <FL/Fl_Hor_Value_Slider.H>
#include <FL/Fl_Float_Input.H>

const double GS_CURVE_PANEL_PtSelectionDistThres = 0.05; //max dist from which a cursor can select a curve point
const double GS_CURVE_PANEL_PtShiftAmount = 0.01;        //the dist a curve point moves when using the arrow keys

class GS_Curve_Panel : public Fl_Group
{
public:
	//definitions

	typedef struct _GS_Curve_Panel_Params
	{
		int width, height;
		static _GS_Curve_Panel_Params Default()
		{
			_GS_Curve_Panel_Params params;
			params.width  = 250;
			params.height = 250;
			return params;
		}
	} GS_Curve_Panel_Params;
	
private:
	//variables	
	Fl_Float_Input *wg_minR_input;
	Fl_Float_Input *wg_maxR_input;

	Fl_Float_Input *wg_in_input;
	Fl_Float_Input *wg_out_input;

	GS_Curve_Vis  *wg_Curve_Vis;

	GS_Curve_Panel_Params params;

	Curve *curvePoints;

	int selectedPtIndex;

public:
	GS_Curve_Panel(Curve *_curvePoints, GS_Curve_Panel_Params _params = GS_Curve_Panel_Params::Default());
	~GS_Curve_Panel();

	GS_Line GetGraphDiag(); 

private:
	void setup_VarDefaults();
	void setup_Widgets();
	void setupResponseInputs();
	void setupInOutInputs();
	void setupCurveVis();
	void setupCurvePoints();

	void resize(int w, int h);

	void paramUpdate();

public: //GS_Curve_Panel-Events
	int handle(int eventID);

private: //GS_Curve_Panel-Events
	bool handle_push(int clickX, int clickY);
	bool handle_release(int clickX, int clickY);
	void release_selection();
	bool handle_drag(int clickX, int clickY);

	bool handle_key_press();
	bool handle_delete_selectedPt();
	void handle_shift_SelectedPt(double xShift, double yShift);

	bool isPointInGraph(int clickX, int clickY,
					    double &graphX, double &graphY);


public: //GS_Curve_Panel-InputEvents

	static void MinResponseChanged(Fl_Widget *, void *GS_Curve_Panel_ptr);
	static void MaxResponseChanged(Fl_Widget *, void *GS_Curve_Panel_ptr);
	static void InValChanged(Fl_Widget *, void *GS_Curve_Panel_ptr);
	static void OutValChanged(Fl_Widget *, void *GS_Curve_Panel_ptr);

private: //GS_Curve_Panel-InputEvents
	void inValChanged();
	void outValChanged();
	void updateInOutInputs();
	void updateResponseInputs();
};

#endif